@@ -10,9 +10,6 @@
 #include "buildrun.h"
 #include "session.h"
 #include "util.h"
-#if HAVE_NSS
-#include "modsign.h"
-#endif
 
 #include <cstdlib>
 #include <fstream>
@@ -59,15 +56,19 @@ run_make_cmd(systemtap_session& s, string& make_cmd)
   if (s.verbose > 2)
     make_cmd += " V=1";
   else if (s.verbose > 1)
-    make_cmd += " >/dev/null";
+    make_cmd += " --no-print-directory";
   else
-    make_cmd += " -s >/dev/null 2>&1";
+    make_cmd += " -s --no-print-directory";
 
-  if (s.verbose > 1) clog << "Running " << make_cmd << endl;
-  rc = stap_system (make_cmd.c_str());
-  if (rc && s.verbose > 1)
-    clog << "Error " << rc << " " << strerror(rc) << endl;
-  return rc;
+  if (strverscmp (s.kernel_base_release.c_str(), "2.6.29") < 0)
+    {
+      // Older kernels, before linux commit #fd54f502841c1, include
+      // gratuitous "echo"s in their Makefile.  We need to suppress
+      // that with this bluntness.
+      make_cmd += " >/dev/null";
+    }
+
+  return stap_system (s.verbose, make_cmd);
 }
 
 static void
@@ -158,6 +159,7 @@ compile_pass (systemtap_session& s)
   output_autoconf(s, o, "autoconf-procfs-owner.c", "STAPCONF_PROCFS_OWNER", NULL);
   output_autoconf(s, o, "autoconf-alloc-percpu-align.c", "STAPCONF_ALLOC_PERCPU_ALIGN", NULL);
   output_autoconf(s, o, "autoconf-x86-gs.c", "STAPCONF_X86_GS", NULL);
+  output_autoconf(s, o, "autoconf-grsecurity.c", "STAPCONF_GRSECURITY", NULL);
 
 #if 0
   /* NB: For now, the performance hit of probe_kernel_read/write (vs. our
@@ -225,20 +227,20 @@ compile_pass (systemtap_session& s)
 
   // Run make
   string make_cmd = string("make")
-    + string (" -C \"") + module_dir + string("\"");
-  make_cmd += string(" M=\"") + s.tmpdir + string("\" modules");
+    + string (" -C \"") + module_dir + string("\""); // XXX: lex_cast_qstring?
+  make_cmd += string(" M=\"") + s.tmpdir + string("\"");
+
+  // Add architecture
+  make_cmd += string(" ARCH=") + lex_cast_qstring(s.architecture);
+
+  // Add any custom kbuild flags
+  for (unsigned k=0; k<s.kbuildflags.size(); k++)
+    make_cmd += string(" ") + lex_cast_qstring(s.kbuildflags[k]);
+
+  make_cmd += string (" modules");
 
   rc = run_make_cmd(s, make_cmd);
 
-#if HAVE_NSS
-  // If a certificate database was specified, then try to sign the module.
-  // Failure to do so is not a fatal error. If the signature is actually needed,
-  // staprun will complain at that time.
-  assert (! s.cert_db_path.empty());
-  if (!rc)
-    sign_module (s);
-#endif
-
   return rc;
 }
 
@@ -252,10 +254,8 @@ kernel_built_uprobes (systemtap_session& s)
 {
   string grep_cmd = string ("/bin/grep -q unregister_uprobe ") + 
     s.kernel_build_tree + string ("/Module.symvers");
-  int rc = stap_system (grep_cmd.c_str());
-  if (rc && s.verbose > 1)
-    clog << "Error " << rc << " " << strerror(rc) << endl;
-  return (rc == 0);
+
+  return (stap_system (s.verbose, grep_cmd) == 0);
 }
 
 static bool
@@ -305,10 +305,8 @@ copy_uprobes_symbols (systemtap_session& s)
   string uprobes_home = s.runtime_path + "/uprobes";
   string cp_cmd = string("/bin/cp ") + uprobes_home +
     string("/Module.symvers ") + s.tmpdir;
-  int rc = stap_system (cp_cmd.c_str());
-  if (rc && s.verbose > 1)
-    clog << "Error " << rc << " " << strerror(rc) << endl;
-  return rc;
+
+  return stap_system (s.verbose, cp_cmd);
 }
 
 static int
@@ -341,8 +339,6 @@ uprobes_pass (systemtap_session& s)
 int
 run_pass (systemtap_session& s)
 {
-  int rc = 0;
-
   // for now, just spawn staprun
   string staprun_cmd = string(getenv("SYSTEMTAP_STAPRUN") ?: BINDIR "/staprun")
     + " "
@@ -354,10 +350,10 @@ run_pass (systemtap_session& s)
     staprun_cmd += "-c " + cmdstr_quoted(s.cmd) + " ";
 
   if (s.target_pid)
-    staprun_cmd += "-t " + stringify(s.target_pid) + " ";
+    staprun_cmd += "-t " + lex_cast(s.target_pid) + " ";
 
   if (s.buffer_size)
-    staprun_cmd += "-b " + stringify(s.buffer_size) + " ";
+    staprun_cmd += "-b " + lex_cast(s.buffer_size) + " ";
 
   if (s.need_uprobes)
     staprun_cmd += "-u ";
@@ -370,12 +366,7 @@ run_pass (systemtap_session& s)
 
   staprun_cmd += s.tmpdir + "/" + s.module_name + ".ko";
 
-  if (s.verbose>1) clog << "Running " << staprun_cmd << endl;
-
-  rc = stap_system (staprun_cmd.c_str ());
-  if (rc && s.verbose > 1)
-    clog << "Error " << rc << " " << strerror(rc) << endl;
-  return rc;
+  return stap_system (s.verbose, staprun_cmd);
 }
 
 
@@ -386,7 +377,7 @@ make_tracequery(systemtap_session& s, string& name,
                 const vector<string>& extra_headers)
 {
   static unsigned tick = 0;
-  string basename("tracequery_kmod_" + lex_cast<string>(++tick));
+  string basename("tracequery_kmod_" + lex_cast(++tick));
 
   // create a subdirectory for the module
   string dir(s.tmpdir + "/" + basename);
@@ -438,6 +429,14 @@ make_tracequery(systemtap_session& s, string& name,
   // make the module
   string make_cmd = "make -C '" + s.kernel_build_tree + "'"
     + " M='" + dir + "' modules";
+
+  // Add architecture
+  make_cmd += string(" ARCH=") + lex_cast_qstring(s.architecture);
+
+  // Add any custom kbuild flags
+  for (unsigned k=0; k<s.kbuildflags.size(); k++)
+    make_cmd += string(" ") + lex_cast_qstring(s.kbuildflags[k]);
+
   if (s.verbose < 4)
     make_cmd += " >/dev/null 2>&1";
   return run_make_cmd(s, make_cmd);
@@ -449,7 +448,7 @@ static int
 make_typequery_kmod(systemtap_session& s, const string& header, string& name)
 {
   static unsigned tick = 0;
-  string basename("typequery_kmod_" + lex_cast<string>(++tick));
+  string basename("typequery_kmod_" + lex_cast(++tick));
 
   // create a subdirectory for the module
   string dir(s.tmpdir + "/" + basename);
@@ -487,6 +486,14 @@ make_typequery_kmod(systemtap_session& s, const string& header, string& name)
   // make the module
   string make_cmd = "make -C '" + s.kernel_build_tree + "'"
     + " M='" + dir + "' modules";
+
+  // Add architecture
+  make_cmd += string(" ARCH=") + lex_cast_qstring(s.architecture);
+
+  // Add any custom kbuild flags
+  for (unsigned k=0; k<s.kbuildflags.size(); k++)
+    make_cmd += string(" ") + lex_cast_qstring(s.kbuildflags[k]);
+
   if (s.verbose < 4)
     make_cmd += " >/dev/null 2>&1";
   return run_make_cmd(s, make_cmd);
@@ -499,7 +506,7 @@ make_typequery_umod(systemtap_session& s, const string& header, string& name)
 {
   static unsigned tick = 0;
 
-  name = s.tmpdir + "/typequery_umod_" + lex_cast<string>(++tick) + ".so";
+  name = s.tmpdir + "/typequery_umod_" + lex_cast(++tick) + ".so";
 
   // make the module
   //
@@ -511,10 +518,7 @@ make_typequery_umod(systemtap_session& s, const string& header, string& name)
      + name + " -xc /dev/null -include " + header;
   if (s.verbose < 4)
     cmd += " >/dev/null 2>&1";
-  int rc = stap_system (cmd.c_str());
-  if (rc && s.verbose > 1)
-    clog << "Error " << rc << " " << strerror(rc) << endl;
-  return rc;
+  return stap_system (s.verbose, cmd);
 }
 
 
