@@ -48,6 +48,10 @@ static void clean_cache(systemtap_session& s);
 void
 add_to_cache(systemtap_session& s)
 {
+  // PR10543: clean the cache *before* we try putting something new into it.
+  // We don't want to risk having the brand new contents being erased again.
+  clean_cache(s);
+
   string stapconf_src_path = s.tmpdir + "/" + s.stapconf_name;
   if (s.verbose > 1)
     clog << "Copying " << stapconf_src_path << " to " << s.stapconf_path << endl;
@@ -71,31 +75,6 @@ add_to_cache(systemtap_session& s)
       return;
     }
 
-#if HAVE_NSS
-  // This is the name of the cached module signature.
-  string module_signature_dest_path = s.hash_path;
-  module_signature_dest_path += ".sgn";
-
-  // Copy the module signature.
-  assert (! s.cert_db_path.empty());
-  string module_signature_src_path = module_src_path;
-  module_signature_src_path += ".sgn";
-
-  STAP_PROBE2(stap, cache__add__nss, module_signature_src_path.c_str(), module_signature_dest_path.c_str());
-
-  if (s.verbose > 1)
-    clog << "Copying " << module_signature_src_path << " to " << module_signature_dest_path << endl;
-  if (copy_file(module_signature_src_path.c_str(), module_signature_dest_path.c_str()) != 0)
-    {
-      cerr << "Copy failed (\"" << module_signature_src_path << "\" to \""
-	   << module_signature_dest_path << "\"): " << strerror(errno) << endl;
-      // NB: this is not so severe as to prevent reuse of the .ko
-      // already copied.
-      //
-      // s.use_cache = false;
-    }
-#endif /* HAVE_NSS */
-
   string c_dest_path = s.hash_path;
   if (c_dest_path.rfind(".ko") == (c_dest_path.size() - 3))
     c_dest_path.resize(c_dest_path.size() - 3);
@@ -115,8 +94,6 @@ add_to_cache(systemtap_session& s)
       //
       // s.use_cache = false;
     }
-
-  clean_cache(s);
 }
 
 
@@ -127,10 +104,6 @@ get_from_cache(systemtap_session& s)
   string module_dest_path = s.tmpdir + "/" + s.module_name + ".ko";
   string c_src_path = s.hash_path;
   int fd_stapconf, fd_module, fd_c;
-#if HAVE_NSS
-  string hash_signature_path = s.hash_path + ".sgn";
-  int fd_signature;
-#endif
 
   if (c_src_path.rfind(".ko") == (c_src_path.size() - 3))
     c_src_path.resize(c_src_path.size() - 3);
@@ -200,24 +173,6 @@ get_from_cache(systemtap_session& s)
 	  close(fd_c);
 	  return false;
 	}
-#if HAVE_NSS
-      // See if module signature exists. It's not an error if it doesn't. It just
-      // means that the module is unsigned.
-      fd_signature = open(hash_signature_path.c_str(), O_RDONLY);
-      if (fd_signature != -1) {
-	string signature_dest_path = module_dest_path + ".sgn";
-	close(fd_signature);
-	if (copy_file(hash_signature_path.c_str(), signature_dest_path.c_str()) != 0)
-	  {
-	    cerr << "Copy failed (\"" << hash_signature_path << "\" to \""
-		 << signature_dest_path << "\"): " << strerror(errno) << endl;
-	    unlink(c_src_path.c_str());
-	    close(fd_module);
-	    close(fd_c);
-	    return false;
-	  }
-      }
-#endif
     }
 
   // We're done with these file handles.
@@ -372,18 +327,6 @@ clean_cache(systemtap_session& s)
     }
 }
 
-// Get the size of a file in bytes
-static size_t
-get_file_size(const string &path)
-{
-  struct stat file_info;
-
-  if (stat(path.c_str(), &file_info) == 0)
-    return file_info.st_size;
-  else
-    return 0;
-}
-
 //Assign a weight for a particular file. A lower weight
 // will be removed before a higher weight.
 //TODO: for now use system mtime... later base a
