@@ -15,6 +15,7 @@
 #include "dwarf_wrappers.h"
 #include "elaborate.h"
 #include "session.h"
+#include "unordered.h"
 
 #include <cstring>
 #include <iostream>
@@ -25,19 +26,14 @@
 
 extern "C" {
 #include <elfutils/libdwfl.h>
-#ifdef HAVE_ELFUTILS_VERSION_H
-  #include <elfutils/version.h>
-  #if !_ELFUTILS_PREREQ(0,142)
-    // Always use newer name, old name is deprecated in 0.142.
-    #define elf_getshdrstrndx elf_getshstrndx
-  #endif
-#else
-  // Really old elfutils version, definitely redefine to use old name.
-  #define elf_getshdrstrndx elf_getshstrndx
-#endif
 #include <regex.h>
 }
 
+#if !_ELFUTILS_PREREQ(0,142)
+// Always use newer name, old name is deprecated in 0.142.
+#define elf_getshdrstrndx elf_getshstrndx
+#endif
+
 
 struct func_info;
 struct inline_instance_info;
@@ -48,21 +44,32 @@ struct dwarf_query;
 enum line_t { ABSOLUTE, RELATIVE, RANGE, WILDCARD };
 enum info_status { info_unknown, info_present, info_absent };
 
-#ifdef HAVE_TR1_UNORDERED_MAP
-#include <tr1/unordered_map>
-template<class T> struct stap_map {
-  typedef std::tr1::unordered_map<std::string, T> type;
-};
-#else
-#include <ext/hash_map>
-template<class T> struct stap_map {
-  typedef __gnu_cxx::hash_map<std::string, T, stap_map> type;
-  size_t operator() (const std::string& s) const
-  { __gnu_cxx::hash<const char*> h; return h(s.c_str()); }
-};
-#endif
-typedef stap_map<Dwarf_Die>::type cu_function_cache_t; // function -> die
-typedef stap_map<cu_function_cache_t*>::type mod_cu_function_cache_t; // module:cu -> function -> die
+// module -> cu die[]
+typedef unordered_map<Dwarf*, std::vector<Dwarf_Die>*> module_cu_cache_t;
+
+// typename -> die
+typedef unordered_map<std::string, Dwarf_Die> cu_type_cache_t;
+
+// cu die -> (typename -> die)
+typedef unordered_map<void*, cu_type_cache_t*> mod_cu_type_cache_t;
+
+// function -> die
+typedef unordered_multimap<std::string, Dwarf_Die> cu_function_cache_t;
+typedef std::pair<cu_function_cache_t::iterator,
+                  cu_function_cache_t::iterator>
+        cu_function_cache_range_t;
+
+// cu die -> (function -> die)
+typedef unordered_map<void*, cu_function_cache_t*> mod_cu_function_cache_t;
+
+// inline function die -> instance die[]
+typedef unordered_map<void*, std::vector<Dwarf_Die>*> cu_inl_function_cache_t;
+
+// die -> parent die
+typedef unordered_map<void*, Dwarf_Die> cu_die_parent_cache_t;
+
+// cu die -> (die -> parent die)
+typedef unordered_map<void*, cu_die_parent_cache_t*> mod_cu_die_parent_cache_t;
 
 typedef std::vector<func_info> func_info_map_t;
 typedef std::vector<inline_instance_info> inline_instance_map_t;
@@ -71,7 +78,6 @@ typedef std::vector<inline_instance_info> inline_instance_map_t;
 /* XXX FIXME functions that dwflpp needs from tapsets.cxx */
 func_info_map_t *get_filtered_functions(dwarf_query *q);
 inline_instance_map_t *get_filtered_inlines(dwarf_query *q);
-void add_label_name(dwarf_query *q, const char *name);
 
 
 struct
@@ -139,6 +145,7 @@ struct inline_instance_info
   {
     std::memset(&die, 0, sizeof(die));
   }
+  bool operator<(const inline_instance_info& other) const;
   std::string name;
   char const * decl_file;
   int decl_line;
@@ -163,7 +170,6 @@ struct dwflpp
   Dwarf_Die * cu;
 
   std::string module_name;
-  std::string cu_name;
   std::string function_name;
 
   dwflpp(systemtap_session & session, const std::string& user_module, bool kernel_p);
@@ -176,6 +182,8 @@ struct dwflpp
   void focus_on_cu(Dwarf_Die * c);
   void focus_on_function(Dwarf_Die * f);
 
+  std::string cu_name(void);
+
   Dwarf_Die *query_cu_containing_address(Dwarf_Addr a);
 
   bool module_name_matches(const std::string& pattern);
@@ -184,7 +192,7 @@ struct dwflpp
 
   bool function_name_matches_pattern(const std::string& name, const std::string& pattern);
   bool function_name_matches(const std::string& pattern);
-  bool function_name_final_match(const std::string& pattern);
+  bool function_scope_matches(const std::vector<std::string> scopes);
 
   void iterate_over_modules(int (* callback)(Dwfl_Module *, void **,
                                              const char *, Dwarf_Addr,
@@ -199,6 +207,10 @@ struct dwflpp
   void iterate_over_inline_instances (int (* callback)(Dwarf_Die * die, void * arg),
                                       void * data);
 
+  std::vector<Dwarf_Die> getscopes_die(Dwarf_Die* die);
+  std::vector<Dwarf_Die> getscopes(Dwarf_Die* die);
+  std::vector<Dwarf_Die> getscopes(Dwarf_Addr pc);
+
   Dwarf_Die *declaration_resolve(const char *name);
 
   mod_cu_function_cache_t cu_function_cache;
@@ -213,13 +225,15 @@ struct dwflpp
                                    enum line_t line_type,
                                    void (* callback) (const dwarf_line_t& line,
                                                       void * arg),
+                                   const std::string& func_pattern,
                                    void *data);
 
   void iterate_over_labels (Dwarf_Die *begin_die,
-                            const char *sym,
-                            const char *symfunction,
-                            void *data,
+                            const std::string& sym,
+                            const std::string& function,
+                            dwarf_query *q,
                             void (* callback)(const std::string &,
+                                              const char *,
                                               const char *,
                                               int,
                                               Dwarf_Die *,
@@ -227,7 +241,7 @@ struct dwflpp
                                               dwarf_query *));
 
   void collect_srcfiles_matching (std::string const & pattern,
-                                  std::set<char const *> & filtered_srcfiles);
+                                  std::set<std::string> & filtered_srcfiles);
 
   void resolve_prologue_endings (func_info_map_t & funcs);
 
@@ -240,7 +254,7 @@ struct dwflpp
 
   bool die_has_pc (Dwarf_Die & die, Dwarf_Addr pc);
 
-  std::string literal_stmt_for_local (Dwarf_Die *scope_die,
+  std::string literal_stmt_for_local (std::vector<Dwarf_Die>& scopes,
                                       Dwarf_Addr pc,
                                       std::string const & local,
                                       const target_symbol *e,
@@ -263,13 +277,10 @@ struct dwflpp
                      const std::string& filename,
                      int line,
                      const std::string& module,
-                     const std::string& section,
                      Dwarf_Addr addr,
                      bool has_return);
 
-  Dwarf_Addr relocate_address(Dwarf_Addr addr,
-                              std::string& reloc_section,
-                              std::string& blacklist_section);
+  Dwarf_Addr relocate_address(Dwarf_Addr addr, std::string& reloc_section);
 
   Dwarf_Addr literal_addr_to_sym_addr(Dwarf_Addr lit_addr);
 
@@ -284,12 +295,17 @@ private:
   void setup_kernel(const std::string& module_name, bool debuginfo_needed = true);
   void setup_user(const std::vector<std::string>& modules, bool debuginfo_needed = true);
 
-  typedef std::map<Dwarf*, std::vector<Dwarf_Die>*> module_cu_cache_t;
   module_cu_cache_t module_cu_cache;
 
-  typedef std::map<std::string, std::vector<Dwarf_Die>*> cu_inl_function_cache_t;
+  std::set<void*> cu_inl_function_cache_done; // CUs that are already cached
   cu_inl_function_cache_t cu_inl_function_cache;
-  static int cu_inl_function_caching_callback (Dwarf_Die* func, void *arg);
+  void cache_inline_instances (Dwarf_Die* die);
+
+  mod_cu_die_parent_cache_t cu_die_parent_cache;
+  void cache_die_parents(cu_die_parent_cache_t* parents, Dwarf_Die* die);
+  cu_die_parent_cache_t *get_die_parents();
+
+  Dwarf_Die* get_parent_scope(Dwarf_Die* die);
 
   /* The global alias cache is used to resolve any DIE found in a
    * module that is stubbed out with DW_AT_declaration with a defining
@@ -299,7 +315,7 @@ private:
    * cache is indexed by name.  If other declaration lookups were
    * added to it, it would have to be indexed by name and tag
    */
-  mod_cu_function_cache_t global_alias_cache;
+  mod_cu_type_cache_t global_alias_cache;
   static int global_alias_caching_callback(Dwarf_Die *die, void *arg);
   int iterate_over_globals (int (* callback)(Dwarf_Die *, void *),
                             void * data);
@@ -316,10 +332,10 @@ private:
   static void loc2c_emit_address (void *arg, struct obstack *pool,
                                   Dwarf_Addr address);
 
-  void print_locals(Dwarf_Die *die, std::ostream &o);
+  void print_locals(std::vector<Dwarf_Die>& scopes, std::ostream &o);
   void print_members(Dwarf_Die *vardie, std::ostream &o);
 
-  Dwarf_Attribute *find_variable_and_frame_base (Dwarf_Die *scope_die,
+  Dwarf_Attribute *find_variable_and_frame_base (std::vector<Dwarf_Die>& scopes,
                                                  Dwarf_Addr pc,
                                                  std::string const & local,
                                                  const target_symbol *e,
@@ -368,15 +384,11 @@ private:
   regex_t blacklist_func; // function/statement probes
   regex_t blacklist_func_ret; // only for .return probes
   regex_t blacklist_file; // file name
+  regex_t blacklist_section; // init/exit sections
   bool blacklist_enabled;
   void build_blacklist();
   std::string get_blacklist_section(Dwarf_Addr addr);
 
-  Dwarf_Addr pc_cached_scopes;
-  int num_cached_scopes;
-  Dwarf_Die *cached_scopes;
-  int dwarf_getscopes_cached (Dwarf_Addr pc, Dwarf_Die **scopes);
-
   // Returns the call frame address operations for the given program counter.
   Dwarf_Op *get_cfa_ops (Dwarf_Addr pc);
 
