@@ -168,13 +168,13 @@ find_script_hash (systemtap_session& s, const string& script, const hash &base)
 
   // Hash user-specified arguments (that change the generated module).
   h.add(s.bulk_mode);			// '-b'
-  h.add(s.merge);			// '-M'
   h.add(s.timing);			// '-t'
   h.add(s.prologue_searching);		// '-P'
   h.add(s.ignore_vmlinux);		// --ignore-vmlinux
   h.add(s.ignore_dwarf);		// --ignore-dwarf
   h.add(s.consult_symtab);		// --kelf, --kmap
   h.add(s.skip_badvars);		// --skip-badvars
+  h.add(s.unprivileged);		// --unprivileged
   if (!s.kernel_symtab_path.empty())	// --kmap
     {
       h.add(s.kernel_symtab_path);
@@ -190,6 +190,10 @@ find_script_hash (systemtap_session& s, const string& script, const hash &base)
   for (unsigned i = 0; i < s.macros.size(); i++)
     h.add(s.macros[i]);
 
+  // Add any custom kbuild flags (-B)
+  for (unsigned i = 0; i < s.kbuildflags.size(); i++)
+    h.add(s.kbuildflags[i]);
+
   // -d MODULE
   for (set<string>::iterator it = s.unwindsym_modules.begin();
        it != s.unwindsym_modules.end();
@@ -233,7 +237,9 @@ find_stapconf_hash (systemtap_session& s, const hash& base)
 {
   hash h(base);
 
-  // The basic hash should be good enough for STAPCONF variables
+  // Add any custom kbuild flags
+  for (unsigned i = 0; i < s.kbuildflags.size(); i++)
+    h.add(s.kbuildflags[i]);
 
   // Get the directory path to store our cached stapconf parameters
   string result, hashdir;
@@ -265,6 +271,10 @@ find_tracequery_hash (systemtap_session& s, const string& header)
   // Add the tracepoint header to the computed hash
   h.add_file(header);
 
+  // Add any custom kbuild flags
+  for (unsigned i = 0; i < s.kbuildflags.size(); i++)
+    h.add(s.kbuildflags[i]);
+
   // Get the directory path to store our cached module
   string result, hashdir;
   h.result(result);
@@ -284,6 +294,11 @@ find_typequery_hash (systemtap_session& s, const string& name)
   // Add the typequery name to distinguish the hash
   h.add(name);
 
+  if (name[0] == 'k')
+    // Add any custom kbuild flags
+    for (unsigned i = 0; i < s.kbuildflags.size(); i++)
+      h.add(s.kbuildflags[i]);
+
   // Get the directory path to store our cached module
   string result, hashdir;
   h.result(result);
