@@ -18,30 +18,35 @@
 #endif
 
 /* Allocated section needs to be writable when creating pic shared objects
-   because we store relocatable addresses in them. */
-#ifdef __PIC__
+   because we store relocatable addresses in them.  We used to make this
+   read only for non-pic executables, but the new semaphore support relies
+   on having a writable .probes section to put the enabled variables in. */
 #define ALLOCSEC "\"aw\""
-#else
-#define ALLOCSEC "\"a\""
-#endif
 
 /* An allocated section .probes that holds the probe names and addrs. */
-#define STAP_PROBE_DATA_(probe,guard,arg)	\
-  __asm__ volatile (".section .probes," ALLOCSEC "\n" \
-		    "\t.align 8\n"		\
-		    "1:\n\t.asciz " #probe "\n" \
-		    "\t.align 4\n"		\
-		    "\t.int " #guard "\n"	\
-  		    "\t.align 8\n"		\
-		    STAP_PROBE_ADDR "1b\n"	\
-  		    "\t.align 8\n"		\
-		    STAP_PROBE_ADDR #arg "\n"	\
-		    "\t.int 0\n"  		\
+#define STAP_PROBE_DATA_(probe,guard,arg)		\
+  __asm__ volatile (".section .probes," ALLOCSEC "\n"	\
+		    "\t.align 8\n"			\
+		    "1:\n\t.asciz " #probe "\n"		\
+		    "\t.align 4\n"			\
+		    "\t.int " #guard "\n"		\
+		    "\t.align 8\n"			\
+		    STAP_PROBE_ADDR "1b\n"		\
+		    "\t.align 8\n"			\
+		    STAP_PROBE_ADDR #arg "\n"		\
+		    "\t.int 0\n"			\
 		    "\t.previous\n")
 
 #define STAP_PROBE_DATA(probe, guard, arg)	\
   STAP_PROBE_DATA_(#probe,guard,arg)
 
+#if defined STAP_HAS_SEMAPHORES && defined EXPERIMENTAL_UTRACE_SDT
+#define STAP_SEMAPHORE(probe)	\
+  if ( probe ## _semaphore )
+#else
+#define STAP_SEMAPHORE(probe)
+#endif
+
 #if ! (defined EXPERIMENTAL_UTRACE_SDT || defined EXPERIMENTAL_KPROBE_SDT)
 
 /* These baroque macros are used to create a unique label. */
@@ -53,7 +58,7 @@
 #else
 #define STAP_COUNTER  STAP_CONCAT(__,LINE__)
 #endif
-#define STAP_LABEL(a,b) STAP_CONCAT(a,b) 
+#define STAP_LABEL(a,b) STAP_CONCAT(a,b)
 
 /* Taking the address of a local label and/or referencing alloca prevents the
    containing function from being inlined, which keeps the parameters visible. */
@@ -65,10 +70,7 @@
 #define STAP_UNINLINE
 #endif
 
-#define STAP_UNINLINE_LABEL(label) \
-  __extension__ static volatile long labelval  __attribute__ ((unused)) = (long) &&label
-
-#if defined __x86_64__ || defined __i386__  || defined __powerpc__
+#if defined __x86_64__ || defined __i386__  || defined __powerpc__ || defined __arm__
 #define STAP_NOP "\tnop "
 #else
 #define STAP_NOP "\tnop 0 "
@@ -77,65 +79,56 @@
 #define STAP_UPROBE_GUARD 0x31425250
 
 #define STAP_PROBE_(probe)			\
-do { \
+do {						\
   STAP_PROBE_DATA(probe,STAP_UPROBE_GUARD,2f);	\
-  __asm__ volatile ("2:\n"	\
-		    STAP_NOP);	\
- } while (0)
+  __asm__ volatile ("2:\n"			\
+		    STAP_NOP);			\
+} while (0)
 
 #define STAP_PROBE1_(probe,label,parm1)			\
-do {							\
-  STAP_UNINLINE_LABEL(label);				\
+do STAP_SEMAPHORE(probe) {				\
   volatile __typeof__((parm1)) arg1 = parm1;		\
   STAP_UNINLINE;					\
   STAP_PROBE_DATA(probe,STAP_UPROBE_GUARD,2f);		\
-  label:						\
   __asm__ volatile ("2:\n"				\
 		    STAP_NOP "/* %0 */" :: "g"(arg1));	\
  } while (0)
 
 #define STAP_PROBE2_(probe,label,parm1,parm2)				\
-do {									\
-  STAP_UNINLINE_LABEL(label);						\
+do STAP_SEMAPHORE(probe) {						\
   volatile __typeof__((parm1)) arg1 = parm1;				\
   volatile __typeof__((parm2)) arg2 = parm2;				\
   STAP_UNINLINE;							\
   STAP_PROBE_DATA(probe,STAP_UPROBE_GUARD,2f);				\
-  label:								\
   __asm__ volatile ("2:\n"						\
 		    STAP_NOP "/* %0 %1 */" :: "g"(arg1), "g"(arg2));	\
 } while (0)
 
 #define STAP_PROBE3_(probe,label,parm1,parm2,parm3)			\
-do {									\
-  STAP_UNINLINE_LABEL(label);						\
+do STAP_SEMAPHORE(probe) {						\
   volatile __typeof__((parm1)) arg1 = parm1;				\
   volatile __typeof__((parm2)) arg2 = parm2;				\
   volatile __typeof__((parm3)) arg3 = parm3;				\
   STAP_UNINLINE;							\
   STAP_PROBE_DATA(probe,STAP_UPROBE_GUARD,2f);				\
-   label:								\
   __asm__ volatile ("2:\n"						\
 		    STAP_NOP "/* %0 %1 %2 */" :: "g"(arg1), "g"(arg2), "g"(arg3)); \
 } while (0)
 
 #define STAP_PROBE4_(probe,label,parm1,parm2,parm3,parm4)		\
-do {									\
-  STAP_UNINLINE_LABEL(label);						\
+do STAP_SEMAPHORE(probe) {						\
   volatile __typeof__((parm1)) arg1 = parm1;				\
   volatile __typeof__((parm2)) arg2 = parm2;				\
   volatile __typeof__((parm3)) arg3 = parm3;				\
   volatile __typeof__((parm4)) arg4 = parm4;				\
   STAP_UNINLINE;							\
   STAP_PROBE_DATA(probe,STAP_UPROBE_GUARD,2f);				\
-  label:								\
   __asm__ volatile ("2:\n"						\
 		    STAP_NOP "/* %0 %1 %2 %3 */" :: "g"(arg1), "g"(arg2), "g"(arg3), "g"(arg4)); \
 } while (0)
 
 #define STAP_PROBE5_(probe,label,parm1,parm2,parm3,parm4,parm5)		\
-do  {									\
-  STAP_UNINLINE_LABEL(label);						\
+do  STAP_SEMAPHORE(probe) {						\
   volatile __typeof__((parm1)) arg1 = parm1;				\
   volatile __typeof__((parm2)) arg2 = parm2;				\
   volatile __typeof__((parm3)) arg3 = parm3;				\
@@ -143,14 +136,12 @@ do  {									\
   volatile __typeof__((parm5)) arg5 = parm5;				\
   STAP_UNINLINE;							\
   STAP_PROBE_DATA(probe,STAP_UPROBE_GUARD,2f);				\
-  label:								\
   __asm__ volatile ("2:\n"						\
 		    STAP_NOP "/* %0 %1 %2 %3 %4 */" :: "g"(arg1), "g"(arg2), "g"(arg3), "g"(arg4), "g"(arg5)); \
 } while (0)
 
 #define STAP_PROBE6_(probe,label,parm1,parm2,parm3,parm4,parm5,parm6)	\
-do {									\
-  STAP_UNINLINE_LABEL(label);						\
+do STAP_SEMAPHORE(probe) {						\
   volatile __typeof__((parm1)) arg1 = parm1;				\
   volatile __typeof__((parm2)) arg2 = parm2;				\
   volatile __typeof__((parm3)) arg3 = parm3;				\
@@ -159,14 +150,12 @@ do {									\
   volatile __typeof__((parm6)) arg6 = parm6;				\
   STAP_UNINLINE;							\
   STAP_PROBE_DATA(probe,STAP_UPROBE_GUARD,2f);				\
-  label:								\
   __asm__ volatile ("2:\n"						\
 		    STAP_NOP "/* %0 %1 %2 %3 %4 %5 */" :: "g"(arg1), "g"(arg2), "g"(arg3), "g"(arg4), "g"(arg5), "g"(arg6)); \
 } while (0)
 
 #define STAP_PROBE7_(probe,label,parm1,parm2,parm3,parm4,parm5,parm6,parm7) \
-do  {									\
-  STAP_UNINLINE_LABEL(label);						\
+do  STAP_SEMAPHORE(probe) {						\
   volatile __typeof__((parm1)) arg1 = parm1;				\
   volatile __typeof__((parm2)) arg2 = parm2;				\
   volatile __typeof__((parm3)) arg3 = parm3;				\
@@ -176,14 +165,12 @@ do  {									\
   volatile __typeof__((parm7)) arg7 = parm7;				\
   STAP_UNINLINE;							\
   STAP_PROBE_DATA(probe,STAP_UPROBE_GUARD,2f);				\
-   label:								\
   __asm__ volatile ("2:\n"						\
 		    STAP_NOP "/* %0 %1 %2 %3 %4 %5 %6 */" :: "g"(arg1), "g"(arg2), "g"(arg3), "g"(arg4), "g"(arg5), "g"(arg6), "g"(arg7)); \
 } while (0)
 
 #define STAP_PROBE8_(probe,label,parm1,parm2,parm3,parm4,parm5,parm6,parm7,parm8) \
-do {									\
-  STAP_UNINLINE_LABEL(label);						\
+do STAP_SEMAPHORE(probe) {						\
   volatile __typeof__((parm1)) arg1 = parm1;				\
   volatile __typeof__((parm2)) arg2 = parm2;				\
   volatile __typeof__((parm3)) arg3 = parm3;				\
@@ -194,14 +181,12 @@ do {									\
   volatile __typeof__((parm8)) arg8 = parm8;				\
   STAP_UNINLINE;							\
   STAP_PROBE_DATA(probe,STAP_UPROBE_GUARD,2f);				\
-   label:								\
   __asm__ volatile ("2:\n"						\
 		    STAP_NOP "/* %0 %1 %2 %3 %4 %5 %6 %7 */" :: "g"(arg1), "g"(arg2), "g"(arg3), "g"(arg4), "g"(arg5), "g"(arg6), "g"(arg7), "g"(arg8)); \
 } while (0)
 
 #define STAP_PROBE9_(probe,label,parm1,parm2,parm3,parm4,parm5,parm6,parm7,parm8,parm9) \
-do {									\
-  STAP_UNINLINE_LABEL(label);						\
+do STAP_SEMAPHORE(probe) {						\
   volatile __typeof__((parm1)) arg1 = parm1;				\
   volatile __typeof__((parm2)) arg2 = parm2;				\
   volatile __typeof__((parm3)) arg3 = parm3;				\
@@ -213,14 +198,12 @@ do {									\
   volatile __typeof__((parm9)) arg9 = parm9;				\
   STAP_UNINLINE;							\
   STAP_PROBE_DATA(probe,STAP_UPROBE_GUARD,2f);				\
-  label:								\
   __asm__ volatile ("2:\n"						\
 		    STAP_NOP "/* %0 %1 %2 %3 %4 %5 %6 %7 %8 */" :: "g"(arg1), "g"(arg2), "g"(arg3), "g"(arg4), "g"(arg5), "g"(arg6), "g"(arg7), "g"(arg8), "g"(arg9)); \
 } while (0)
 
 #define STAP_PROBE10_(probe,label,parm1,parm2,parm3,parm4,parm5,parm6,parm7,parm8,parm9,parm10) \
-do {									\
-  STAP_UNINLINE_LABEL(label);						\
+do STAP_SEMAPHORE(probe) {						\
   volatile __typeof__((parm1)) arg1 = parm1;				\
   volatile __typeof__((parm2)) arg2 = parm2;				\
   volatile __typeof__((parm3)) arg3 = parm3;				\
@@ -233,7 +216,6 @@ do {									\
   volatile __typeof__((parm10)) arg10 = parm10;				\
   STAP_UNINLINE;							\
   STAP_PROBE_DATA(probe,STAP_UPROBE_GUARD,2f);				\
-  label:								\
   __asm__ volatile ("2:\n"						\
 		    STAP_NOP "/* %0 %1 %2 %3 %4 %5 %6 %7 %8 %9 */" :: "g"(arg1), "g"(arg2), "g"(arg3), "g"(arg4), "g"(arg5), "g"(arg6), "g"(arg7), "g"(arg8), "g"(arg9), "g"(arg10)); \
 } while (0)
@@ -255,50 +237,50 @@ extern long int syscall (long int __sysno, ...) __THROW;
 #include <sys/syscall.h>
 
 #define STAP_PROBE_(probe)			\
-do {						\
-  STAP_PROBE_DATA(probe,STAP_SYSCALL,0);	\
+do STAP_SEMAPHORE(probe) {			\
+  STAP_PROBE_DATA(probe,STAP_GUARD,0);		\
   syscall (STAP_SYSCALL, #probe, STAP_GUARD);	\
  } while (0)
 
 #define STAP_PROBE1_(probe,label,parm1)				\
-do {								\
+do STAP_SEMAPHORE(probe) {					\
   STAP_PROBE_DATA(probe,STAP_GUARD,1);				\
   syscall (STAP_SYSCALL, #probe, STAP_GUARD, (size_t)parm1);	\
  } while (0)
 
 #define STAP_PROBE2_(probe,label,parm1,parm2)				\
-do {									\
+do STAP_SEMAPHORE(probe) {						\
   __extension__ struct {size_t arg1 __attribute__((aligned(8)));	\
 	  size_t arg2 __attribute__((aligned(8)));}			\
   stap_probe2_args = {(size_t)parm1, (size_t)parm2};			\
   STAP_PROBE_DATA(probe,STAP_GUARD,2);					\
-  syscall (STAP_SYSCALL, #probe, STAP_GUARD, &stap_probe2_args);		\
+  syscall (STAP_SYSCALL, #probe, STAP_GUARD, &stap_probe2_args);	\
  } while (0)
 
 #define STAP_PROBE3_(probe,label,parm1,parm2,parm3)			\
-do {									\
+do STAP_SEMAPHORE(probe) {						\
   __extension__ struct {size_t arg1 __attribute__((aligned(8)));	\
 	  size_t arg2 __attribute__((aligned(8)));			\
 	  size_t arg3 __attribute__((aligned(8)));}			\
   stap_probe3_args = {(size_t)parm1, (size_t)parm2, (size_t)parm3};	\
   STAP_PROBE_DATA(probe,STAP_GUARD,3);					\
-  syscall (STAP_SYSCALL, #probe, STAP_GUARD, &stap_probe3_args);		\
+  syscall (STAP_SYSCALL, #probe, STAP_GUARD, &stap_probe3_args);	\
  } while (0)
 
 #define STAP_PROBE4_(probe,label,parm1,parm2,parm3,parm4)		\
-do {									\
+do STAP_SEMAPHORE(probe) {						\
   __extension__ struct {size_t arg1 __attribute__((aligned(8)));	\
 	  size_t arg2 __attribute__((aligned(8)));			\
 	  size_t arg3 __attribute__((aligned(8)));			\
 	  size_t arg4 __attribute__((aligned(8)));}			\
   stap_probe4_args = {(size_t)parm1, (size_t)parm2, (size_t)parm3, (size_t)parm4}; \
   STAP_PROBE_DATA(probe,STAP_GUARD,4);					\
-  syscall (STAP_SYSCALL, #probe, STAP_GUARD,&stap_probe4_args);		\
+  syscall (STAP_SYSCALL, #probe, STAP_GUARD,&stap_probe4_args);	\
  } while (0)
 
 #define STAP_PROBE5_(probe,label,parm1,parm2,parm3,parm4,parm5)		\
-do {									\
-  __extension__ struct {size_t arg1 __attribute__((aligned(8)));			\
+do STAP_SEMAPHORE(probe) {						\
+  __extension__ struct {size_t arg1 __attribute__((aligned(8)));	\
 	  size_t arg2 __attribute__((aligned(8)));			\
 	  size_t arg3 __attribute__((aligned(8)));			\
 	  size_t arg4 __attribute__((aligned(8)));			\
@@ -306,12 +288,12 @@ do {									\
   stap_probe5_args = {(size_t)parm1, (size_t)parm2, (size_t)parm3, (size_t)parm4, \
 	(size_t)parm5};							\
   STAP_PROBE_DATA(probe,STAP_GUARD,5);					\
-  syscall (STAP_SYSCALL, #probe, STAP_GUARD, &stap_probe5_args);		\
+  syscall (STAP_SYSCALL, #probe, STAP_GUARD, &stap_probe5_args);	\
  } while (0)
 
 #define STAP_PROBE6_(probe,label,parm1,parm2,parm3,parm4,parm5,parm6)	\
-do {									\
-  __extension__ struct {size_t arg1 __attribute__((aligned(8)));			\
+do STAP_SEMAPHORE(probe) {						\
+  __extension__ struct {size_t arg1 __attribute__((aligned(8)));	\
 	  size_t arg2 __attribute__((aligned(8)));			\
 	  size_t arg3 __attribute__((aligned(8)));			\
 	  size_t arg4 __attribute__((aligned(8)));			\
@@ -320,12 +302,12 @@ do {									\
   stap_probe6_args = {(size_t)parm1, (size_t)parm2, (size_t)parm3, (size_t)parm4, \
 	(size_t)parm5, (size_t)parm6};					\
   STAP_PROBE_DATA(probe,STAP_GUARD,6);					\
-  syscall (STAP_SYSCALL, #probe, STAP_GUARD, &stap_probe6_args);		\
+  syscall (STAP_SYSCALL, #probe, STAP_GUARD, &stap_probe6_args);	\
  } while (0)
 
 #define STAP_PROBE7_(probe,label,parm1,parm2,parm3,parm4,parm5,parm6,parm7) \
-do {									\
-  __extension__ struct {size_t arg1 __attribute__((aligned(8)));			\
+do STAP_SEMAPHORE(probe) {						\
+  __extension__ struct {size_t arg1 __attribute__((aligned(8)));	\
 	  size_t arg2 __attribute__((aligned(8)));			\
 	  size_t arg3 __attribute__((aligned(8)));			\
 	  size_t arg4 __attribute__((aligned(8)));			\
@@ -335,12 +317,12 @@ do {									\
   stap_probe7_args = {(size_t)parm1, (size_t)parm2, (size_t)parm3, (size_t)parm4, \
 	(size_t)parm5, (size_t)parm6, (size_t)parm7};			\
   STAP_PROBE_DATA(probe,STAP_GUARD,7);					\
-  syscall (STAP_SYSCALL, #probe, STAP_GUARD, &stap_probe7_args);		\
+  syscall (STAP_SYSCALL, #probe, STAP_GUARD, &stap_probe7_args);	\
  } while (0)
 
 #define STAP_PROBE8_(probe,label,parm1,parm2,parm3,parm4,parm5,parm6,parm7,parm8) \
-do {									\
-  __extension__ struct {size_t arg1 __attribute__((aligned(8)));			\
+do STAP_SEMAPHORE(probe) {						\
+  __extension__ struct {size_t arg1 __attribute__((aligned(8)));	\
 	  size_t arg2 __attribute__((aligned(8)));			\
 	  size_t arg3 __attribute__((aligned(8)));			\
 	  size_t arg4 __attribute__((aligned(8)));			\
@@ -351,12 +333,12 @@ do {									\
   stap_probe8_args = {(size_t)parm1, (size_t)parm2, (size_t)parm3, (size_t)parm4, \
 	(size_t)parm5, (size_t)parm6, (size_t)parm7, (size_t)parm8};	\
   STAP_PROBE_DATA(probe,STAP_GUARD,8);					\
-  syscall (STAP_SYSCALL, #probe, STAP_GUARD, &stap_probe8_args);		\
+  syscall (STAP_SYSCALL, #probe, STAP_GUARD, &stap_probe8_args);	\
  } while (0)
 
 #define STAP_PROBE9_(probe,label,parm1,parm2,parm3,parm4,parm5,parm6,parm7,parm8,parm9) \
-do {									\
-  __extension__ struct {size_t arg1 __attribute__((aligned(8)));			\
+do STAP_SEMAPHORE(probe) {						\
+  __extension__ struct {size_t arg1 __attribute__((aligned(8)));	\
 	  size_t arg2 __attribute__((aligned(8)));			\
 	  size_t arg3 __attribute__((aligned(8)));			\
 	  size_t arg4 __attribute__((aligned(8)));			\
@@ -368,12 +350,12 @@ do {									\
   stap_probe9_args = {(size_t)parm1, (size_t)parm2, (size_t)parm3, (size_t)parm4, \
 	(size_t)parm5, (size_t)parm6, (size_t)parm7, (size_t)parm8, (size_t)parm9}; \
   STAP_PROBE_DATA(probe,STAP_GUARD,9);					\
-  syscall (STAP_SYSCALL, #probe, STAP_GUARD, &stap_probe9_args);		\
+  syscall (STAP_SYSCALL, #probe, STAP_GUARD, &stap_probe9_args);	\
  } while (0)
 
 #define STAP_PROBE10_(probe,label,parm1,parm2,parm3,parm4,parm5,parm6,parm7,parm8,parm9,parm10) \
-do {									\
-  __extension__ struct {size_t arg1 __attribute__((aligned(8)));			\
+do STAP_SEMAPHORE(probe) {						\
+  __extension__ struct {size_t arg1 __attribute__((aligned(8)));	\
 	  size_t arg2 __attribute__((aligned(8)));			\
 	  size_t arg3 __attribute__((aligned(8)));			\
 	  size_t arg4 __attribute__((aligned(8)));			\
@@ -386,7 +368,7 @@ do {									\
   stap_probe10_args = {(size_t)parm1, (size_t)parm2, (size_t)parm3, (size_t)parm4, \
 	(size_t)parm5, (size_t)parm6, (size_t)parm7, (size_t)parm8, (size_t)parm9, (size_t)parm10}; \
   STAP_PROBE_DATA(probe,STAP_GUARD,10);					\
-  syscall (STAP_SYSCALL, #probe, STAP_GUARD, &stap_probe10_args);		\
+  syscall (STAP_SYSCALL, #probe, STAP_GUARD, &stap_probe10_args);	\
  } while (0)
 
 #endif
@@ -421,19 +403,19 @@ STAP_PROBE1(provider,probe,parm1)
 #define DTRACE_PROBE2(provider,probe,parm1,parm2) \
 STAP_PROBE2(provider,probe,parm1,parm2)
 #define DTRACE_PROBE3(provider,probe,parm1,parm2,parm3) \
-STAP_PROBE3(provider,probe,parm1,parm2,parm3) 
+STAP_PROBE3(provider,probe,parm1,parm2,parm3)
 #define DTRACE_PROBE4(provider,probe,parm1,parm2,parm3,parm4) \
-STAP_PROBE4(provider,probe,parm1,parm2,parm3,parm4) 
+STAP_PROBE4(provider,probe,parm1,parm2,parm3,parm4)
 #define DTRACE_PROBE5(provider,probe,parm1,parm2,parm3,parm4,parm5) \
-STAP_PROBE5(provider,probe,parm1,parm2,parm3,parm4,parm5) 
+STAP_PROBE5(provider,probe,parm1,parm2,parm3,parm4,parm5)
 #define DTRACE_PROBE6(provider,probe,parm1,parm2,parm3,parm4,parm5,parm6) \
-STAP_PROBE6(provider,probe,parm1,parm2,parm3,parm4,parm5,parm6) 
+STAP_PROBE6(provider,probe,parm1,parm2,parm3,parm4,parm5,parm6)
 #define DTRACE_PROBE7(provider,probe,parm1,parm2,parm3,parm4,parm5,parm6,parm7) \
-STAP_PROBE7(provider,probe,parm1,parm2,parm3,parm4,parm5,parm6,parm7) 
+STAP_PROBE7(provider,probe,parm1,parm2,parm3,parm4,parm5,parm6,parm7)
 #define DTRACE_PROBE8(provider,probe,parm1,parm2,parm3,parm4,parm5,parm6,parm7,parm8) \
-STAP_PROBE8(provider,probe,parm1,parm2,parm3,parm4,parm5,parm6,parm7,parm8) 
+STAP_PROBE8(provider,probe,parm1,parm2,parm3,parm4,parm5,parm6,parm7,parm8)
 #define DTRACE_PROBE9(provider,probe,parm1,parm2,parm3,parm4,parm5,parm6,parm7,parm8,parm9) \
-STAP_PROBE9(provider,probe,parm1,parm2,parm3,parm4,parm5,parm6,parm7,parm8,parm9) 
+STAP_PROBE9(provider,probe,parm1,parm2,parm3,parm4,parm5,parm6,parm7,parm8,parm9)
 
 #endif /* sys/sdt.h */
 
