@@ -21,16 +21,13 @@
 #include <stdarg.h>
 #include "loc2c.h"
 
+#if !defined(_ELFUTILS_PREREQ)
+// make a dummy PREREQ check for elfutils < 0.138
+#define _ELFUTILS_PREREQ(major, minor) (0 >= 1)
+#endif
+
 #define _(msg) msg
 
-static const char *
-dwarf_diename_integrate (Dwarf_Die *die)
-{
-  Dwarf_Attribute attr_mem;
-  return dwarf_formstring (dwarf_attr_integrate (die, DW_AT_name, &attr_mem));
-}
-
-
 static void __attribute__ ((noreturn))
 fail (void *arg __attribute__ ((unused)), const char *fmt, ...)
 {
@@ -76,7 +73,7 @@ get_location (Dwarf_Addr dwbias, Dwarf_Addr pc, Dwarf_Attribute *loc_attr,
 }
 
 static void
-handle_variable (Dwarf_Die *scopes, int nscopes, int out,
+handle_variable (Dwarf_Die *lscopes, int lnscopes, int out,
 		 Dwarf_Addr cubias, Dwarf_Die *vardie, Dwarf_Addr pc,
 		 Dwarf_Op *cfa_ops, char **fields)
 {
@@ -85,13 +82,18 @@ handle_variable (Dwarf_Die *scopes, int nscopes, int out,
   struct obstack pool;
   obstack_init (&pool);
 
-   /* Figure out the appropriate frame base for accessing this variable.
-     XXX not handling nested functions
-     XXX inlines botched
-  */
+  /* Figure out the appropriate frame base for accessing this variable.
+   * XXX not handling nested functions
+   */
   Dwarf_Attribute fb_attr_mem, *fb_attr = NULL;
   int inner;
-  for (inner = 0; inner < nscopes; ++inner)
+  /* We start out walking the "lexical scopes" as returned by
+   * as returned by dwarf_getscopes for the address, starting with the
+   * 'out' scope that the variable was found in.
+   */
+  Dwarf_Die *scopes = lscopes;
+  int nscopes = lnscopes;
+  for (inner = out; inner < nscopes && fb_attr == NULL; ++inner)
     {
       switch (dwarf_tag (&scopes[inner]))
 	{
@@ -99,14 +101,25 @@ handle_variable (Dwarf_Die *scopes, int nscopes, int out,
 	  continue;
 	case DW_TAG_subprogram:
 	case DW_TAG_entry_point:
-	case DW_TAG_inlined_subroutine:	/* XXX */
-	  if (inner >= out)
-	    fb_attr = dwarf_attr_integrate (&scopes[inner],
-					    DW_AT_frame_base,
-					    &fb_attr_mem);
+	  fb_attr = dwarf_attr_integrate (&scopes[inner],
+					  DW_AT_frame_base,
+					  &fb_attr_mem);
+	  break;
+	case DW_TAG_inlined_subroutine:
+	  /* Unless we already are going through the "pyshical die tree",
+	   * we now need to start walking the die tree where this
+	   * subroutine is inlined to find the appropriate frame base. */
+          if (out != -1)
+	    {
+	      nscopes = dwarf_getscopes_die (&scopes[inner], &scopes);
+	      if (nscopes == -1)
+		error (2, 0, _("cannot get die scopes inlined_subroutine: %s"),
+		       dwarf_errmsg (-1));
+	      inner = 0; // zero is current scope, for look will increase.
+	      out = -1;
+	    }
 	  break;
 	}
-      break;
     }
 
   Dwarf_Attribute attr_mem;
@@ -123,7 +136,7 @@ handle_variable (Dwarf_Die *scopes, int nscopes, int out,
 
   struct location *head, *tail = NULL;
   head = c_translate_location (&pool, &fail, NULL, NULL,
-			       1, cubias, pc, locexpr, locexpr_len,
+			       1, cubias, pc, &attr_mem, locexpr, locexpr_len,
 			       &tail, fb_attr, cfa_ops);
 
   if (dwarf_attr_integrate (vardie, DW_AT_type, &attr_mem) == NULL)
@@ -187,13 +200,13 @@ handle_variable (Dwarf_Die *scopes, int nscopes, int out,
 	    {
 	    case 1:		/* No children.  */
 	      error (2, 0, _("empty struct %s"),
-		     dwarf_diename_integrate (die) ?: "<anonymous>");
+		     dwarf_diename (die) ?: "<anonymous>");
 	      break;
 	    case -1:		/* Error.  */
 	    default:		/* Shouldn't happen */
 	      error (2, 0, _("%s %s: %s"),
 		     typetag == DW_TAG_union_type ? "union" : "struct",
-		     dwarf_diename_integrate (die) ?: "<anonymous>",
+		     dwarf_diename (die) ?: "<anonymous>",
 		     dwarf_errmsg (-1));
 	      break;
 
@@ -201,7 +214,7 @@ handle_variable (Dwarf_Die *scopes, int nscopes, int out,
 	      break;
 	    }
 	  while (dwarf_tag (die) != DW_TAG_member
-		 || ({ const char *member = dwarf_diename_integrate (die);
+		 || ({ const char *member = dwarf_diename (die);
 		       member == NULL || strcmp (member, *fields); }))
 	    if (dwarf_siblingof (die, &die_mem) != 0)
 	      error (2, 0, _("field name %s not found"), *fields);
@@ -217,34 +230,26 @@ handle_variable (Dwarf_Die *scopes, int nscopes, int out,
 	    }
 	  else
 	    {
-	      /* We are expection a block, constant or loclistptr. */
-	      unsigned int form = dwarf_whatform (&attr_mem);
-	      Dwarf_Sword off;
-	      switch (form)
+	      /* We expect a block or a constant.  In older elfutils,
+		 dwarf_getlocation_addr would not handle the constant for
+		 us, but newer ones do.  For older ones, we work around
+		 it by faking an expression, which is what newer ones do.  */
+#if !_ELFUTILS_PREREQ (0,142)
+	      Dwarf_Op offset_loc = { .atom = DW_OP_plus_uconst };
+	      if (dwarf_formudata (&attr_mem, &offset_loc.number) == 0)
+		c_translate_location (&pool, NULL, NULL, NULL,
+				      1, cubias, pc, &attr_mem,
+				      &offset_loc, 1,
+				      &tail, NULL, NULL);
+	      else
+#endif
 		{
-		/* constant */
-		case DW_FORM_data1:
-		case DW_FORM_data2:
-		case DW_FORM_sdata:
-		case DW_FORM_udata:
-		  if (dwarf_formsdata (&attr_mem, &off) != 0)
-		    error (2, 0, _("Bad offset for %s %s: %s"),
-			   typetag == DW_TAG_union_type ? "union" : "struct",
-			   dwarf_diename_integrate (die) ?: "<anonymous>",
-			   dwarf_errmsg (-1));
-		    if (off != 0)
-		      c_translate_add_offset (&pool, 1,
-					      dwarf_diename_integrate (die)
-					      ?: "", off, &tail);
-		  break;
-
-		default:
-		    locexpr = get_location (cubias, pc, &attr_mem,
-					    &locexpr_len);
-		    c_translate_location (&pool, NULL, NULL, NULL,
-					  1, cubias, pc, locexpr, locexpr_len,
-					  &tail, NULL, NULL);
-		    break;
+		  locexpr = get_location (cubias, pc, &attr_mem,
+					  &locexpr_len);
+		  c_translate_location (&pool, NULL, NULL, NULL,
+					1, cubias, pc, &attr_mem,
+					locexpr, locexpr_len,
+					&tail, NULL, NULL);
 		}
 	    }
 	  ++fields;
@@ -252,7 +257,7 @@ handle_variable (Dwarf_Die *scopes, int nscopes, int out,
 
 	case DW_TAG_base_type:
 	  error (2, 0, _("field %s vs base type %s"),
-		 *fields, dwarf_diename_integrate (die) ?: "<anonymous type>");
+		 *fields, dwarf_diename (die) ?: "<anonymous type>");
 	  break;
 
 	case -1:
@@ -261,7 +266,7 @@ handle_variable (Dwarf_Die *scopes, int nscopes, int out,
 
 	default:
 	  error (2, 0, _("%s: unexpected type tag %#x"),
-		 dwarf_diename_integrate (die) ?: "<anonymous type>",
+		 dwarf_diename (die) ?: "<anonymous type>",
 		 dwarf_tag (die));
 	  break;
 	}
@@ -301,6 +306,7 @@ handle_variable (Dwarf_Die *scopes, int nscopes, int out,
       break;
 
     case DW_TAG_pointer_type:
+    case DW_TAG_reference_type:
       if (store)
 	error (2, 0, _("store not supported for pointer type"));
       c_translate_pointer (&pool, 1, cubias, typedie, &tail);
@@ -474,7 +480,8 @@ main (int argc, char **argv)
 	  Dwarf_Die *const die = &scopes[n];
 
 	  indent += INDENT;
-	  printf ("%*s%s (%#x)", indent, "",
+	  printf ("%*s[%6" PRIx64 "] %s (%#x)", indent, "",
+		  dwarf_dieoffset (die),
 		  dwarf_diename (die) ?: "<unnamed>",
 		  dwarf_tag (die));
 
@@ -527,7 +534,6 @@ main (int argc, char **argv)
 	{
 	  Dwarf_Op *cfa_ops = NULL;
 
-#ifdef _ELFUTILS_PREREQ
 #if _ELFUTILS_PREREQ(0,142)
 	  size_t cfa_nops;
 	  Dwarf_Addr bias;
@@ -554,7 +560,6 @@ main (int argc, char **argv)
 		}
 	    }
 #endif
-#endif	  
 
 	  handle_variable (scopes, n, out, cubias, &vardie, pc, cfa_ops,
 			   &argv[argi]);
