@@ -21,6 +21,8 @@
 #include "coveragedb.h"
 #include "git_version.h"
 #include "rpm_finder.h"
+#include "task_finder.h"
+
 #include "sys/sdt.h"
 
 #include <iostream>
@@ -45,6 +47,7 @@ extern "C" {
 
 using namespace std;
 
+#define PATH_TBD string("__TBD__")
 
 void
 version ()
@@ -103,11 +106,13 @@ usage (systemtap_session& s, int exitcode)
     clog << "              " << s.include_path[i] << endl;
   clog
     << "   -D NM=VAL  emit macro definition into generated C code" << endl
+    << "   -B NM=VAL  pass option to kbuild make" << endl
     << "   -R DIR     look in DIR for runtime, instead of" << endl
     << "              " << s.runtime_path << endl
     << "   -r DIR     cross-compile to kernel with given build tree; or else" << endl
     << "   -r RELEASE cross-compile to kernel /lib/modules/RELEASE/build, instead of" << endl
     << "              " << s.kernel_build_tree << endl
+    << "   -a ARCH    cross-compile to given architecture, instead of " << s.architecture << endl
     << "   -m MODULE  set probe module name, instead of " << endl
     << "              " << s.module_name << endl
     << "   -o FILE    send script output to file, instead of stdout. This supports" << endl
@@ -132,6 +137,8 @@ usage (systemtap_session& s, int exitcode)
 #ifdef HAVE_LIBSQLITE3
     << "   -q         generate information on tapset coverage" << endl
 #endif /* HAVE_LIBSQLITE3 */
+    << "   --unprivileged" << endl
+    << "              restrict usage to features available to unprivileged users" << endl
 #if 0 /* PR6864: disable temporarily; should merge with -d somehow */
     << "   --kelf     make do with symbol table from vmlinux" << endl
     << "   --kmap[=FILE]" << endl
@@ -356,6 +363,42 @@ setup_kernel_release (systemtap_session &s, const char* kstr) {
       }
 }
 
+static void
+checkOptions (systemtap_session &s)
+{
+  bool optionsConflict = false;
+
+  if((s.cmd != "") && (s.target_pid))
+    {
+      cerr << "You can't specify -c and -x options together." <<endl;
+      optionsConflict = true;
+    }
+
+  if (s.unprivileged)
+    {
+      if (s.guru_mode)
+	{
+	  cerr << "You can't specify -g and --unprivileged together." << endl;
+	  optionsConflict = true;
+	}
+    }
+
+  if (!s.kernel_symtab_path.empty())
+    {
+      if (s.consult_symtab)
+      {
+        cerr << "You can't specify --kelf and --kmap together." << endl;
+	optionsConflict = true;
+      }
+      s.consult_symtab = true;
+      if (s.kernel_symtab_path == PATH_TBD)
+        s.kernel_symtab_path = string("/boot/System.map-") + s.kernel_release;
+    }
+
+  if (optionsConflict)
+    usage (s, 1);
+}
+
 int
 main (int argc, char * const argv [])
 {
@@ -371,7 +414,25 @@ main (int argc, char * const argv [])
   s.kernel_release = string (buf.release);
   s.kernel_build_tree = "/lib/modules/" + s.kernel_release + "/build";
 
-  s.architecture = string (buf.machine);
+  // PR4186: Copy logic from coreutils uname (uname -i) to squash
+  // i?86->i386.  Actually, copy logic from linux top-level Makefile
+  // to squash uname -m -> $(SUBARCH).
+
+  string machine = buf.machine;
+  if (machine == "i486") machine = "i386";
+  else if (machine == "i586") machine = "i386";
+  else if (machine == "i686") machine = "i386";
+  else if (machine == "sun4u") machine = "sparc64";
+  else if (machine.substr(0,3) == "arm") machine = "arm";
+  else if (machine == "sa110") machine = "arm";
+  else if (machine == "s390x") machine = "s390";
+  else if (machine.substr(0,3) == "ppc") machine = "powerpc";
+  else if (machine.substr(0,4) == "mips") machine = "mips";
+  else if (machine.substr(0,3) == "sh2") machine = "sh";
+  else if (machine.substr(0,3) == "sh3") machine = "sh";
+  else if (machine.substr(0,3) == "sh4") machine = "sh";
+
+  s.architecture = machine;
   for (unsigned i=0; i<5; i++) s.perpass_verbose[i]=0;
   s.timing = false;
   s.guru_mode = false;
@@ -389,13 +450,12 @@ main (int argc, char * const argv [])
 
   s.buffer_size = 0;
   s.last_pass = 5;
-  s.module_name = "stap_" + stringify(getpid());
-  s.stapconf_name = "stapconf_" + stringify(getpid()) + ".h";
+  s.module_name = "stap_" + lex_cast(getpid());
+  s.stapconf_name = "stapconf_" + lex_cast(getpid()) + ".h";
   s.output_file = ""; // -o FILE
   s.keep_tmpdir = false;
   s.cmd = "";
   s.target_pid = 0;
-  s.merge=true;
   s.perfmon=0;
   s.symtab = false;
   s.use_cache = true;
@@ -406,11 +466,12 @@ main (int argc, char * const argv [])
   s.ignore_dwarf = false;
   s.load_only = false;
   s.skip_badvars = false;
+  s.unprivileged = false;
 
   // Location of our signing certificate.
   // If we're root, use the database in SYSCONFDIR, otherwise
   // use the one in our $HOME directory.  */
-  if (geteuid() == 0)
+  if (getuid() == 0)
     s.cert_db_path = SYSCONFDIR "/systemtap/ssl/server";
   else
     s.cert_db_path = getenv("HOME") + string ("/.systemtap/ssl/server");
@@ -460,6 +521,14 @@ main (int argc, char * const argv [])
 	}
     }
 
+  // Location of our signing certificate.
+  // If we're root, use the database in SYSCONFDIR, otherwise
+  // use the one in s.data_path.  */
+  if (geteuid() == 0)
+    s.cert_db_path = SYSCONFDIR "/systemtap/ssl/server";
+  else
+    s.cert_db_path = s.data_path + "/ssl/server";
+
   const char* s_tc = getenv ("SYSTEMTAP_COVERAGE");
   if (s_tc != NULL)
     s.tapset_compile_coverage = true;
@@ -480,6 +549,7 @@ main (int argc, char * const argv [])
 #define LONG_OPT_IGNORE_DWARF 4
 #define LONG_OPT_VERBOSE_PASS 5
 #define LONG_OPT_SKIP_BADVARS 6
+#define LONG_OPT_UNPRIVILEGED 7
       // NB: also see find_hash(), usage(), switch stmt below, stap.1 man page
       static struct option long_options[] = {
         { "kelf", 0, &long_opt, LONG_OPT_KELF },
@@ -488,10 +558,11 @@ main (int argc, char * const argv [])
         { "ignore-dwarf", 0, &long_opt, LONG_OPT_IGNORE_DWARF },
 	{ "skip-badvars", 0, &long_opt, LONG_OPT_SKIP_BADVARS },
         { "vp", 1, &long_opt, LONG_OPT_VERBOSE_PASS },
+        { "unprivileged", 0, &long_opt, LONG_OPT_UNPRIVILEGED },
         { NULL, 0, NULL, 0 }
       };
-      int grc = getopt_long (argc, argv, "hVMvtp:I:e:o:R:r:m:kgPc:x:D:bs:uqwl:d:L:FS:",
-                                                          long_options, NULL);
+      int grc = getopt_long (argc, argv, "hVvtp:I:e:o:R:r:a:m:kgPc:x:D:bs:uqwl:d:L:FS:B:",
+                             long_options, NULL);
       if (grc < 0)
         break;
       switch (grc)
@@ -500,10 +571,6 @@ main (int argc, char * const argv [])
           version ();
           exit (0);
 
-        case 'M':
-          s.merge = false;
-          break;
-
         case 'v':
           for (unsigned i=0; i<5; i++)
             s.perpass_verbose[i] ++;
@@ -518,17 +585,17 @@ main (int argc, char * const argv [])
 	  break;
 
         case 'p':
-          if (s.listing_mode)
-            {
-              cerr << "Listing (-l) mode implies pass 2." << endl;
-              usage (s, 1);
-            }
           s.last_pass = (int)strtoul(optarg, &num_endptr, 10);
           if (*num_endptr != '\0' || s.last_pass < 1 || s.last_pass > 5)
             {
               cerr << "Invalid pass number (should be 1-5)." << endl;
               usage (s, 1);
             }
+          if (s.listing_mode && s.last_pass != 2)
+            {
+              cerr << "Listing (-l) mode implies pass 2." << endl;
+              usage (s, 1);
+            }
           break;
 
         case 'I':
@@ -537,6 +604,9 @@ main (int argc, char * const argv [])
 
         case 'd':
           s.unwindsym_modules.insert (string (optarg));
+          // PR10228: trigger task-finder logic early if -d /USER-MODULE/ given.
+          if (optarg[0] == '/')
+            enable_task_finder (s);
           break;
 
         case 'e':
@@ -608,6 +678,10 @@ main (int argc, char * const argv [])
           setup_kernel_release(s, optarg);
           break;
 
+        case 'a':
+          s.architecture = string(optarg);
+          break;
+
         case 'k':
           s.keep_tmpdir = true;
           s.use_cache = false; /* User wants to keep a usable build tree. */
@@ -689,6 +763,10 @@ main (int argc, char * const argv [])
           s.load_only = true;
 	  break;
 
+	case 'B':
+          s.kbuildflags.push_back (string (optarg));
+	  break;
+
         case 0:
           switch (long_opt)
             {
@@ -705,7 +783,6 @@ main (int argc, char * const argv [])
               if (optarg)
                 s.kernel_symtab_path = optarg;
               else
-#define PATH_TBD string("__TBD__")
                 s.kernel_symtab_path = PATH_TBD;
 	      break;
 	    case LONG_OPT_IGNORE_VMLINUX:
@@ -737,6 +814,9 @@ main (int argc, char * const argv [])
 	    case LONG_OPT_SKIP_BADVARS:
 	      s.skip_badvars = true;
 	      break;
+	    case LONG_OPT_UNPRIVILEGED:
+	      s.unprivileged = true;
+	      break;
             default:
               cerr << "Internal error parsing command arguments." << endl;
               usage(s, 1);
@@ -749,40 +829,13 @@ main (int argc, char * const argv [])
         }
     }
 
-  if(!s.bulk_mode && !s.merge)
-    {
-      cerr << "-M option is valid only for bulk (relayfs) mode." <<endl;
-      usage (s, 1);
-    }
-
-  if(!s.output_file.empty() && s.bulk_mode && !s.merge)
-    {
-      cerr << "You can't specify -M, -b and -o options together." <<endl;
-      usage (s, 1);
-    }
-
-  if((s.cmd != "") && (s.target_pid))
-    {
-      cerr << "You can't specify -c and -x options together." <<endl;
-      usage (s, 1);
-    }
-
-  if (!s.kernel_symtab_path.empty())
-    {
-      if (s.consult_symtab)
-      {
-        cerr << "You can't specify --kelf and --kmap together." << endl;
-	usage (s, 1);
-      }
-      s.consult_symtab = true;
-      if (s.kernel_symtab_path == PATH_TBD)
-        s.kernel_symtab_path = string("/boot/System.map-") + s.kernel_release;
-    }
+  // Check for options conflicts.
+  checkOptions (s);
 
   // Warn in case the target kernel release doesn't match the running one.
   if (s.last_pass > 4 &&
       (string(buf.release) != s.kernel_release ||
-       string(buf.machine) != s.architecture))
+       machine != s.architecture)) // NB: squashed ARCH by PR4186 logic
    {
      if(! s.suppress_warnings)
        cerr << "WARNING: kernel release/architecture mismatch with host forces last-pass 4." << endl;
@@ -1151,7 +1204,7 @@ main (int argc, char * const argv [])
          << endl;
   else
     {
-      // Update cache. Cache cleaning is kicked off at the end of this function.
+      // Update cache. Cache cleaning is kicked off at the beginning of this function.
       if (s.use_cache)
         add_to_cache(s);
 
@@ -1172,20 +1225,6 @@ main (int argc, char * const argv [])
 	  if (copy_file(module_src_path.c_str(), module_dest_path.c_str()) != 0)
 	    cerr << "Copy failed (\"" << module_src_path << "\" to \""
 		 << module_dest_path << "\"): " << strerror(errno) << endl;
-
-#if HAVE_NSS
-	  // Save the signature as well.
-	  assert (! s.cert_db_path.empty());
-	  module_src_path += ".sgn";
-	  module_dest_path += ".sgn";
-
-	  if (s.verbose > 1)
-	    clog << "Copying " << module_src_path << " to "
-		 << module_dest_path << endl;
-	  if (copy_file(module_src_path.c_str(), module_dest_path.c_str()) != 0)
-	    cerr << "Copy failed (\"" << module_src_path << "\" to \""
-		 << module_dest_path << "\"): " << strerror(errno) << endl;
-#endif
 	}
     }
 
@@ -1249,10 +1288,8 @@ pass_5:
 	  // Remove the temporary directory.
           string cleanupcmd = "rm -rf ";
           cleanupcmd += s.tmpdir;
-          if (s.verbose>1) clog << "Running " << cleanupcmd << endl;
-	  int status = stap_system (cleanupcmd.c_str());
-	  if (status != 0 && s.verbose>1)
-	    clog << "Cleanup command failed, status: " << status << endl;
+
+	  (void) stap_system (s.verbose, cleanupcmd);
         }
     }
 
