@@ -1,6 +1,6 @@
 /* -*- linux-c -*- 
  * I/O for printing warnings, errors and debug messages
- * Copyright (C) 2005, 2006, 2007 Red Hat Inc.
+ * Copyright (C) 2005-2009 Red Hat Inc.
  *
  * This file is part of systemtap, and is free software.  You can
  * redistribute it and/or modify it under the terms of the GNU General
@@ -22,14 +22,17 @@
 #define ERR_STRING "ERROR: "
 enum code { INFO=0, WARN, ERROR, DBUG };
 
+static void _stp_vlog (enum code type, const char *func, int line, const char *fmt, va_list args)
+        __attribute ((format (printf, 4, 0)));
+
 static void _stp_vlog (enum code type, const char *func, int line, const char *fmt, va_list args)
 {
 	int num;
-	char *buf = per_cpu_ptr(Stp_lbuf, smp_processor_id());
+	char *buf = per_cpu_ptr(Stp_lbuf, get_cpu());
 	int start = 0;
 
 	if (type == DBUG) {
-		start = _stp_snprintf(buf, STP_LOG_BUF_LEN, "\033[36m%s:%d:\033[0m ", func, line);
+		start = _stp_snprintf(buf, STP_LOG_BUF_LEN, "%s:%d: ", func, line);
 	} else if (type == WARN) {
 		strcpy (buf, WARN_STRING);
 		start = sizeof(WARN_STRING) - 1;
@@ -38,7 +41,7 @@ static void _stp_vlog (enum code type, const char *func, int line, const char *f
 		start = sizeof(ERR_STRING) - 1;
 	}
 
-	num = _stp_vscnprintf (buf + start, STP_LOG_BUF_LEN - start - 1, fmt, args);
+	num = vscnprintf (buf + start, STP_LOG_BUF_LEN - start - 1, fmt, args);
 	if (num + start) {
 		if (buf[num + start - 1] != '\n') {
 			buf[num + start] = '\n';
@@ -46,31 +49,21 @@ static void _stp_vlog (enum code type, const char *func, int line, const char *f
 			buf[num + start] = '\0';
 		}
 
+#ifdef STAP_DEBUG_PRINTK
+                if (type == DBUG) printk (KERN_DEBUG "%s", buf);
+                else if (type == WARN) printk (KERN_WARNING "%s", buf);
+                else if (type == ERROR) printk (KERN_ERR "%s", buf);
+                else printk (KERN_INFO "%s", buf);
+#else
 		if (type != DBUG)
 			_stp_ctl_write(STP_OOB_DATA, buf, start + num + 1);
 		else {
 			_stp_print(buf);
 			_stp_print_flush();
 		}
+#endif
 	}
-}
-
-/** Logs Data.
- * This function sends the message immediately to staprun. It
- * will also be sent over the bulk transport (relayfs) if it is
- * being used. If the last character is not a newline, then one 
- * is added. This function is not as efficient as _stp_printf()
- * and should only be used for urgent messages. You probably want
- * dbug(), or _stp_warn().
- * @param fmt A variable number of args.
- * @todo Evaluate if this function is necessary.
- */
-void _stp_log (const char *fmt, ...)
-{
-	va_list args;
-	va_start(args, fmt);
-	_stp_vlog (INFO, NULL, 0, fmt, args);
-	va_end(args);
+	put_cpu();
 }
 
 /** Prints warning.
@@ -80,7 +73,7 @@ void _stp_log (const char *fmt, ...)
  * is added. 
  * @param fmt A variable number of args.
  */
-void _stp_warn (const char *fmt, ...)
+static void _stp_warn (const char *fmt, ...)
 {
 	va_list args;
 	va_start(args, fmt);
@@ -96,7 +89,7 @@ void _stp_warn (const char *fmt, ...)
  * call. You should probably call return immediately after 
  * calling _stp_exit().
  */
-void _stp_exit (void)
+static void _stp_exit (void)
 {
 	_stp_exit_flag = 1;
 }
@@ -111,7 +104,7 @@ void _stp_exit (void)
  * @param fmt A variable number of args.
  * @sa _stp_exit().
  */
-void _stp_error (const char *fmt, ...)
+static void _stp_error (const char *fmt, ...)
 {
 	va_list args;
 	va_start(args, fmt);
@@ -130,7 +123,7 @@ void _stp_error (const char *fmt, ...)
  * @param fmt A variable number of args.
  * @sa _stp_error
  */
-void _stp_softerror (const char *fmt, ...)
+static void _stp_softerror (const char *fmt, ...)
 {
 	va_list args;
 	va_start(args, fmt);
