@@ -1,5 +1,5 @@
 /* common register includes used in multiple modules
- * Copyright (C) 2005 Red Hat Inc.
+ * Copyright (C) 2005-2008 Red Hat Inc.
  * Copyright (C) 2005 Intel Corporation.
  *
  * This file is part of systemtap, and is free software.  You can
@@ -12,8 +12,10 @@
 #define _REGS_H_
 
 #if defined  (STAPCONF_X86_UNIREGS) && (defined (__x86_64__) || defined (__i386__))
+
 #define REG_IP(regs) regs->ip
 #define REG_SP(regs) regs->sp
+#define REG_FP(regs) regs->bp
 
 #elif defined  (__x86_64__)
 
@@ -24,10 +26,15 @@
 
 #define REG_IP(regs) regs->eip
 #define REG_SP(regs) regs->esp
+#define REG_FP(regs) regs->ebp
 
 #elif defined (__ia64__)
+
 #define REG_IP(regs)    ((regs)->cr_iip +ia64_psr(regs)->ri)
 #define REG_SP(regs)    ((regs)->r12)
+#define SET_REG_IP(regs, x) \
+  (((regs)->cr_iip = (x) & ~3UL), (ia64_psr(regs)->ri = (x) & 3UL))
+
 
 #elif defined (__powerpc64__)
 
@@ -42,16 +49,24 @@
 #define REG_LINK(regs) regs->ARM_lr
 
 #elif defined (__s390__) || defined (__s390x__)
+
 #ifndef __s390x__
 #define PSW_ADDR_AMODE	0x80000000UL
+#define PSW_ADDR_INSN	0x7FFFFFFFUL
 #else /* __s390x__ */
 #define PSW_ADDR_AMODE	0x0000000000000000UL
+#define PSW_ADDR_INSN	0xFFFFFFFFFFFFFFFFUL
 #endif /* __s390x__ */
-#define REG_IP(regs)	(((regs)->psw.addr) | PSW_ADDR_AMODE)
+#define REG_IP(regs)	(((regs)->psw.addr) & PSW_ADDR_INSN)
 #define REG_SP(regs)	(regs)->gprs[15]
+#define SET_REG_IP(regs,x) (regs)->psw.addr = (x) | PSW_ADDR_AMODE
 
 #else
 #error "Unimplemented architecture"
 #endif
 
+#ifndef SET_REG_IP
+#define SET_REG_IP(regs, x) REG_IP(regs) = x
+#endif
+
 #endif /* _REGS_H_ */
