@@ -134,7 +134,6 @@ const char *moderror(int err);
 int insert_module(const char *path, const char *special_options,
 	char **options);
 int mountfs(void);
-int check_permissions(void);
 void start_symbol_thread(void);
 void stop_symbol_thread(void);
 
@@ -171,6 +170,7 @@ extern int need_uprobes;
 extern int daemon_mode;
 extern off_t fsize_max;
 extern int fnum_max;
+extern int unprivileged_user;
 
 /* getopt variables */
 extern char *optarg;
@@ -186,3 +186,13 @@ extern int out_fd[NR_CPUS];
 /* relay_old uses these. Set in ctl.c */
 extern unsigned subbuf_size;
 extern unsigned n_subbufs;
+
+/*
+ * ppoll exists in glibc >= 2.4
+ */
+#if (__GLIBC__ < 2) || ((__GLIBC__ == 2) && (__GLIBC_MINOR__ < 4))
+#define NEED_PPOLL
+
+extern int ppoll(struct pollfd *fds, nfds_t nfds,
+		 const struct timespec *timeout, const sigset_t *sigmask);
+#endif
