@@ -4,6 +4,9 @@
 #if ! defined(CONFIG_UTRACE)
 /* Dummy definitions for use in sym.c */
 struct stap_task_finder_target { };
+static int stap_start_task_finder(void) { return 0; }
+static void stap_stop_task_finder(void) { }
+
 #else
 
 #include <linux/utrace.h>
@@ -16,7 +19,9 @@ struct stap_task_finder_target { };
 #include <linux/list.h>
 #include <linux/binfmts.h>
 #include <linux/mount.h>
-
+#ifndef STAPCONF_TASK_UID
+#include <linux/cred.h>
+#endif
 #include "syscall.h"
 #include "utrace_compatibility.h"
 #include "task_finder_map.c"
@@ -40,18 +45,10 @@ static atomic_t __stp_attach_count = ATOMIC_INIT (0);
 
 #define debug_task_finder_attach() (atomic_inc(&__stp_attach_count))
 #define debug_task_finder_detach() (atomic_dec(&__stp_attach_count))
-#ifdef DEBUG_TASK_FINDER_PRINTK
-#define debug_task_finder_report() (printk(KERN_ERR \
-					   "%s:%d attach count: %d, inuse count: %d\n", \
-					   __FUNCTION__, __LINE__,	\
-					   atomic_read(&__stp_attach_count), \
-					   atomic_read(&__stp_inuse_count)))
-#else
 #define debug_task_finder_report() (_stp_dbug(__FUNCTION__, __LINE__, \
 					      "attach count: %d, inuse count: %d\n", \
 					      atomic_read(&__stp_attach_count), \
 					      atomic_read(&__stp_inuse_count)))
-#endif	/* !DEBUG_TASK_FINDER_PRINTK */
 #else
 #define debug_task_finder_attach()	/* empty */
 #define debug_task_finder_detach()	/* empty */
@@ -97,7 +94,7 @@ struct stap_task_finder_target {
 	size_t pathlen;
 
 /* public: */
-	const char *pathname;
+	const char *procname;
 	pid_t pid;
 	stap_task_finder_callback callback;
 	stap_task_finder_mmap_callback mmap_callback;
@@ -167,8 +164,8 @@ stap_register_task_finder_target(struct stap_task_finder_target *new_tgt)
 	if (new_tgt == NULL)
 		return EFAULT;
 
-	if (new_tgt->pathname != NULL)
-		new_tgt->pathlen = strlen(new_tgt->pathname);
+	if (new_tgt->procname != NULL)
+		new_tgt->pathlen = strlen(new_tgt->procname);
 	else
 		new_tgt->pathlen = 0;
 
@@ -185,14 +182,14 @@ stap_register_task_finder_target(struct stap_task_finder_target *new_tgt)
 	new_tgt->ops.report_syscall_exit = \
 		&__stp_utrace_task_finder_target_syscall_exit;
 
-	// Search the list for an existing entry for pathname/pid.
+	// Search the list for an existing entry for procname/pid.
 	list_for_each(node, &__stp_task_finder_list) {
 		tgt = list_entry(node, struct stap_task_finder_target, list);
 		if (tgt != NULL
-		    /* pathname-based target */
+		    /* procname-based target */
 		    && ((new_tgt->pathlen > 0
 			 && tgt->pathlen == new_tgt->pathlen
-			 && strcmp(tgt->pathname, new_tgt->pathname) == 0)
+			 && strcmp(tgt->procname, new_tgt->procname) == 0)
 			/* pid-based target (a specific pid or all
 			 * pids) */
 			|| (new_tgt->pathlen == 0 && tgt->pathlen == 0
@@ -727,22 +724,28 @@ __stp_utrace_attach_match_filename(struct task_struct *tsk,
 	size_t filelen;
 	struct list_head *tgt_node;
 	struct stap_task_finder_target *tgt;
+	uid_t tsk_euid;
 
+#ifdef STAPCONF_TASK_UID
+	tsk_euid = tsk->euid;
+#else
+	tsk_euid = task_euid(tsk);
+#endif
 	filelen = strlen(filename);
 	list_for_each(tgt_node, &__stp_task_finder_list) {
 		int rc;
 
 		tgt = list_entry(tgt_node, struct stap_task_finder_target,
 				 list);
-		// If we've got a matching pathname or we're probing
+		// If we've got a matching procname or we're probing
 		// all threads, we've got a match.  We've got to keep
 		// matching since a single thread could match a
-		// pathname and match an "all thread" probe.
+		// procname and match an "all thread" probe.
 		if (tgt == NULL)
 			continue;
 		else if (tgt->pathlen > 0
 			 && (tgt->pathlen != filelen
-			     || strcmp(tgt->pathname, filename) != 0))
+			     || strcmp(tgt->procname, filename) != 0))
 			continue;
 		/* Ignore pid-based target, they were handled at startup. */
 		else if (tgt->pid != 0)
@@ -750,6 +753,18 @@ __stp_utrace_attach_match_filename(struct task_struct *tsk,
 		/* Notice that "pid == 0" (which means to probe all
 		 * threads) falls through. */
 
+#ifndef STP_PRIVILEGED
+		/* Make sure unprivileged users only probe their own threads. */
+		if (_stp_uid != tsk_euid) {
+			if (tgt->pid != 0) {
+				_stp_warn("Process %d does not belong to unprivileged user %d",
+					  tsk->pid, _stp_uid);
+			}
+			continue;
+		}
+#endif
+
+
 		// Set up events we need for attached tasks. When
 		// register_p is set, we won't actually call the
 		// callbacks here - we'll call it when the thread gets
@@ -780,8 +795,8 @@ __stp_utrace_attach_match_filename(struct task_struct *tsk,
 }
 
 // This function handles the details of getting a task's associated
-// pathname, and calling __stp_utrace_attach_match_filename() to
-// attach to it if we find the pathname "interesting".  So, what's the
+// procname, and calling __stp_utrace_attach_match_filename() to
+// attach to it if we find the procname "interesting".  So, what's the
 // difference between path_tsk and match_tsk?  Normally they are the
 // same, except in one case.  In an UTRACE_EVENT(EXEC), we need to
 // detach engines from the newly exec'ed process (since its path has
@@ -1332,8 +1347,8 @@ stap_start_task_finder(void)
 	int rc = 0;
 	struct task_struct *grp, *tsk;
 	char *mmpath_buf;
+	uid_t tsk_euid;
 
-	debug_task_finder_report();
 	mmpath_buf = _stp_kmalloc(PATH_MAX);
 	if (mmpath_buf == NULL) {
 		_stp_error("Unable to allocate space for path");
@@ -1392,6 +1407,11 @@ stap_start_task_finder(void)
 		}
 
 		/* Check the thread's exe's path/pid against our list. */
+#ifdef STAPCONF_TASK_UID
+		tsk_euid = tsk->euid;
+#else
+		tsk_euid = task_euid(tsk);
+#endif
 		mmpathlen = strlen(mmpath);
 		list_for_each(tgt_node, &__stp_task_finder_list) {
 			struct stap_task_finder_target *tgt;
@@ -1400,10 +1420,10 @@ stap_start_task_finder(void)
 					 struct stap_task_finder_target, list);
 			if (tgt == NULL)
 				continue;
-			/* pathname-based target */
+			/* procname-based target */
 			else if (tgt->pathlen > 0
 				 && (tgt->pathlen != mmpathlen
-				     || strcmp(tgt->pathname, mmpath) != 0))
+				     || strcmp(tgt->procname, mmpath) != 0))
 				 continue;
 			/* pid-based target */
 			else if (tgt->pid != 0 && tgt->pid != tsk->pid)
@@ -1411,6 +1431,17 @@ stap_start_task_finder(void)
 			/* Notice that "pid == 0" (which means to
 			 * probe all threads) falls through. */
 
+#ifndef STP_PRIVILEGED
+			/* Make sure unprivileged users only probe their own threads.  */
+			if (_stp_uid != tsk_euid) {
+				if (tgt->pid != 0 || _stp_target) {
+					_stp_warn("Process %d does not belong to unprivileged user %d",
+						  tsk->pid, _stp_uid);
+				}
+				continue;
+			}
+#endif
+
 			// Set up events we need for attached tasks.
 			rc = __stp_utrace_attach(tsk, &tgt->ops, tgt,
 						 __STP_ATTACHED_TASK_EVENTS,
@@ -1422,8 +1453,8 @@ stap_start_task_finder(void)
 	} while_each_thread(grp, tsk);
 stf_err:
 	rcu_read_unlock();
-
 	_stp_kfree(mmpath_buf);
+	debug_task_finder_report(); // report at end for utrace engine counting
 	return rc;
 }
 
