@@ -54,10 +54,10 @@ typedef struct __stp_time_t {
     struct timer_list timer;
 } stp_time_t;
 
-void *stp_time = NULL;
+static void *stp_time = NULL;
 
 /* Flag to tell the timer callback whether to reregister */
-int stp_timer_reregister = 0;
+static int stp_timer_reregister = 0;
 
 /* Try to estimate the number of CPU cycles in a millisecond - i.e. kHz.  This
  * relies heavily on the accuracy of udelay.  By calling udelay twice, we
@@ -185,7 +185,7 @@ __stp_time_cpufreq_callback(struct notifier_block *self,
     return NOTIFY_OK;
 }
 
-struct notifier_block __stp_time_notifier = {
+static struct notifier_block __stp_time_notifier = {
     .notifier_call = __stp_time_cpufreq_callback,
 };
 
@@ -205,7 +205,7 @@ __stp_constant_freq(void)
 #endif /* CONFIG_CPU_FREQ */
 
 /* This function is called during module unloading. */
-void
+static void
 _stp_kill_time(void)
 {
     if (stp_time) {
@@ -223,21 +223,28 @@ _stp_kill_time(void)
 #endif
 
         _stp_free_percpu(stp_time);
+        stp_time = NULL;
     }
 }
 
 /* This function is called during module loading. */
-int
+static int
 _stp_init_time(void)
 {
     int ret = 0;
 
+    _stp_kill_time();
+
     stp_time = _stp_alloc_percpu(sizeof(stp_time_t));
     if (unlikely(stp_time == 0))
 	    return -1;
     
     stp_timer_reregister = 1;
+#ifdef STAPCONF_ONEACHCPU_RETRY
     ret = on_each_cpu(__stp_init_time, NULL, 0, 1);
+#else
+    ret = on_each_cpu(__stp_init_time, NULL, 1);
+#endif
 
 #ifdef CONFIG_CPU_FREQ
     if (!ret && !__stp_constant_freq()) {
@@ -259,12 +266,12 @@ _stp_init_time(void)
         }
     }
 #endif
-   if (ret)
+    if (ret)
         _stp_kill_time();
     return ret;
 }
 
-int64_t
+static int64_t
 _stp_gettimeofday_ns(void)
 {
     int64_t base;
@@ -274,6 +281,9 @@ _stp_gettimeofday_ns(void)
     stp_time_t *time;
     int i = 0;
 
+    if (!stp_time)
+        return -1;
+
     preempt_disable();
     time = per_cpu_ptr(stp_time, smp_processor_id());
 
@@ -292,7 +302,6 @@ _stp_gettimeofday_ns(void)
     }
 
     delta = get_cycles() - last;
-
     preempt_enable_no_resched();
 
 #if defined (__s390__) || defined (__s390x__)
@@ -313,6 +322,8 @@ _stp_gettimeofday_ns(void)
     // Verify units:
     //   (D cycles) * (1E6 ns/ms) / (F cycles/ms [kHz]) = ns
     delta *= NSEC_PER_MSEC;
+    if (freq == 0)
+      return 0;
     do_div(delta, freq);
 #endif
 
