@@ -173,6 +173,9 @@ static int _stp_ctl_send(int type, void *data, int len)
 		msleep(5);
 	if (err > 0)
 		wake_up_interruptible(&_stp_ctl_wq);
+        else
+                // printk instead of _stp_error since an error here means our transport is suspect
+                printk(KERN_ERR "ctl_send (type=%d len=%d) failed: %d\n", type, len, err);
 	dbug_trans(1, "returning %d\n", err);
 	return err;
 }
