@@ -17,6 +17,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  * Copyright (C) IBM Corporation, 2006
+ * Copyright (C) Red Hat, Inc. 2009
  */
 #include <linux/types.h>
 #include <linux/hash.h>
@@ -28,6 +29,8 @@
 #include <linux/kref.h>
 #include <linux/utrace.h>
 #include <linux/regset.h>
+#include <linux/file.h>
+#include <linux/version.h>
 #define UPROBES_IMPLEMENTATION 1
 
 /* PR9974: Adapt to struct renaming. */
@@ -955,10 +958,15 @@ static int defer_registration(struct uprobe *u, int regflag,
  */
 static struct pid *uprobe_get_tg_leader(pid_t p)
 {
-	struct pid *pid;
+	struct pid *pid = NULL;
 
 	rcu_read_lock();
-	pid = find_vpid(p);
+	/*
+	 * We need this check because unmap_u[ret]probe() can be called
+	 * from a report_death callback, where current->proxy is NULL.
+	 */
+	if (current->nsproxy)
+		pid = find_vpid(p);
 	if (pid) {
 		struct task_struct *t = pid_task(pid, PIDTYPE_PID);
 		if (t)
@@ -1138,8 +1146,7 @@ fail_tsk:
 }
 EXPORT_SYMBOL_GPL(register_uprobe);
 
-/* See Documentation/uprobes.txt. */
-void unregister_uprobe(struct uprobe *u)
+void __unregister_uprobe(struct uprobe *u, bool remove_bkpt)
 {
 	struct pid *p;
 	struct uprobe_process *uproc;
@@ -1193,10 +1200,13 @@ void unregister_uprobe(struct uprobe *u)
 	if (!list_empty(&ppt->uprobe_list))
 		goto done;
 
-	/*
-	 * The last uprobe at ppt's probepoint is being unregistered.
-	 * Queue the breakpoint for removal.
-	 */
+	/* The last uprobe at ppt's probepoint is being unregistered. */
+	if (!remove_bkpt) {
+		uprobe_free_probept(ppt);
+		goto done;
+	}
+
+	/* Queue the breakpoint for removal. */
 	ppt->state = UPROBE_REMOVING;
 	list_add_tail(&ppt->pd_node, &uproc->pending_uprobes);
 
@@ -1221,8 +1231,20 @@ done:
 	up_write(&uproc->rwsem);
 	uprobe_put_process(uproc, false);
 }
+
+/* See Documentation/uprobes.txt. */
+void unregister_uprobe(struct uprobe *u)
+{
+	__unregister_uprobe(u, true);
+}
 EXPORT_SYMBOL_GPL(unregister_uprobe);
 
+void unmap_uprobe(struct uprobe *u)
+{
+	__unregister_uprobe(u, false);
+}
+EXPORT_SYMBOL_GPL(unmap_uprobe);
+
 /* Find a surviving thread in uproc.  Runs with uproc->rwsem locked. */
 static struct task_struct *find_surviving_thread(struct uprobe_process *uproc)
 {
@@ -1368,6 +1390,9 @@ static noinline unsigned long uprobe_setup_ssol_vma(unsigned long nbytes)
 	unsigned long addr;
 	struct mm_struct *mm;
 	struct vm_area_struct *vma;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,28)
+	struct file *file;
+#endif
 
 	BUG_ON(nbytes & ~PAGE_MASK);
 	if ((addr = find_old_ssol_vma()) != 0)
@@ -1381,17 +1406,40 @@ static noinline unsigned long uprobe_setup_ssol_vma(unsigned long nbytes)
 	 * Find the end of the top mapping and skip a page.
 	 * If there is no space for PAGE_SIZE above
 	 * that, mmap will ignore our address hint.
+	 *
+	 * We allocate a "fake" unlinked shmem file because anonymous
+	 * memory might not be granted execute permission when the selinux
+	 * security hooks have their way. Only do this for 2.6.28 or higher
+	 * since shmem_file_setup() isn't exported before that.
 	 */
 	vma = rb_entry(rb_last(&mm->mm_rb), struct vm_area_struct, vm_rb);
 	addr = vma->vm_end + PAGE_SIZE;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,28)
+	file = shmem_file_setup("uprobes/ssol", nbytes, VM_NORESERVE);
+	if (file) {
+		addr = do_mmap_pgoff(file, addr, nbytes, PROT_EXEC,
+				     MAP_PRIVATE, 0);
+		fput(file);
+	}
+	if (!file || addr & ~PAGE_MASK) {
+#else
 	addr = do_mmap_pgoff(NULL, addr, nbytes, PROT_EXEC,
 					MAP_PRIVATE|MAP_ANONYMOUS, 0);
 	if (addr & ~PAGE_MASK) {
+#endif
 		up_write(&mm->mmap_sem);
 		mmput(mm);
-		printk(KERN_ERR "Uprobes failed to allocate a vma for"
-			" pid/tgid %d/%d for single-stepping out of line.\n",
-			current->pid, current->tgid);
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,28)
+		if (!file)
+			printk(KERN_ERR "Uprobes shmem_file_setup failed while"
+				  " allocating vma for pid/tgid %d/%d for"
+				  " single-stepping out of line.\n",
+				  current->pid, current->tgid);
+		else
+#endif
+			printk(KERN_ERR "Uprobes failed to allocate a vma for"
+				" pid/tgid %d/%d for single-stepping out of"
+				" line.\n", current->pid, current->tgid);
 		return addr;
 	}
 
@@ -2220,7 +2268,8 @@ static u32 uprobe_report_exit(enum utrace_resume_action action,
 			}
 		}
 		up_read(&uproc->rwsem);
-		if (utask->state == UPTASK_TRAMPOLINE_HIT)
+		if (utask->state == UPTASK_TRAMPOLINE_HIT ||
+						utask->state == UPTASK_BP_HIT)
 			uprobe_decref_process(uproc);
 	}
 
@@ -2718,6 +2767,14 @@ void unregister_uretprobe(struct uretprobe *rp)
 }
 EXPORT_SYMBOL_GPL(unregister_uretprobe);
 
+void unmap_uretprobe(struct uretprobe *rp)
+{
+	if (!rp)
+		return;
+	unmap_uprobe(&rp->u);
+}
+EXPORT_SYMBOL_GPL(unmap_uretprobe);
+
 /*
  * uproc->ssol_area has been successfully set up.  Establish the
  * uretprobe trampoline in the next available slot following the
