@@ -83,6 +83,7 @@ struct systemtap_session
   std::vector<std::string> include_path;
   std::vector<std::string> macros;
   std::vector<std::string> args;
+  std::vector<std::string> kbuildflags;
   std::string kernel_release;
   std::string kernel_base_release;
   std::string kernel_build_tree;
@@ -106,7 +107,6 @@ struct systemtap_session
   bool listing_mode_vars;
   bool bulk_mode;
   bool unoptimized;
-  bool merge;
   bool suppress_warnings;
   int buffer_size;
   unsigned perfmon;
@@ -115,6 +115,7 @@ struct systemtap_session
   bool tapset_compile_coverage;
   bool need_uprobes;
   bool load_only; // flight recorder mode
+  bool unprivileged;
 
   // NB: It is very important for all of the above (and below) fields
   // to be cleared in the systemtap_session ctor (elaborate.cxx)
@@ -220,6 +221,10 @@ struct systemtap_session
   void print_error_source (std::ostream&, std::string&, const token* tok);
   void print_warning (const std::string& w, const token* tok = 0);
 
+
+  // Location of semaphores to activate sdt probes
+  std::map<derived_probe*, Dwarf_Addr> sdt_semaphore_addr;
+
   // NB: It is very important for all of the above (and below) fields
   // to be cleared in the systemtap_session ctor (elaborate.cxx)
   // and/or main.cxx(main).
