@@ -41,6 +41,7 @@ static char             *dbdir   = NULL;
 static char requestFileName[] = "/tmp/stap.server.client.zip.XXXXXX";
 static char responseDirName[] = "/tmp/stap.server.XXXXXX";
 static char responseZipName[] = "/tmp/stap.server.XXXXXX.zip.XXXXXX";
+static const char *stapOptions = "";
 
 static void
 Usage(const char *progName)
@@ -94,7 +95,7 @@ readDataFromSocket(PRFileDesc *sslSocket)
       return SECFailure;
     }
 
-  /* Read the number fo bytes to be received.  */
+  /* Read the number of bytes to be received.  */
   numBytesRead = PR_Read(sslSocket, & info.size, sizeof (info.size));
   if (numBytesRead == 0) /* EOF */
     {
@@ -377,6 +378,7 @@ handle_connection(PRFileDesc *tcpSocket)
   PRSocketOptionData socketOption;
   PRFileInfo         info;
   char              *cmdline;
+  char              *stap_server_prefix;
   int                rc;
   char              *rc1;
 
@@ -465,19 +467,23 @@ handle_connection(PRFileDesc *tcpSocket)
 #endif
 
   /* Call the stap-server script.  */
-  cmdline = PORT_Alloc(sizeof ("stap-server") +
-		       sizeof (requestFileName) +
-		       sizeof (responseDirName) +
-		       sizeof (responseZipName) +
-		       strlen (dbdir) + 1);
+  stap_server_prefix = getenv("SYSTEMTAP_SERVER_SCRIPTS") ?: BINDIR;
+  cmdline = PORT_Alloc(strlen (stap_server_prefix) + sizeof ("/stap-server") + 1 +
+		       sizeof (requestFileName) + 1 +
+		       sizeof (responseDirName) + 1 +
+		       sizeof (responseZipName) + 1 +
+		       strlen (dbdir) + 1 +
+		       1 + strlen (stapOptions) + 1 +
+		       1);
   if (! cmdline) {
     errWarn ("PORT_Alloc");
     secStatus = SECFailure;
     goto cleanup;
   }
 
-  sprintf (cmdline, "stap-server %s %s %s %s",
-	   requestFileName, responseDirName, responseZipName, dbdir);
+  sprintf (cmdline, "%s/stap-server %s %s %s %s '%s'", stap_server_prefix,
+	   requestFileName, responseDirName, responseZipName, dbdir,
+	   stapOptions);
   rc = system (cmdline);
 
   PR_Free (cmdline);
@@ -701,25 +707,26 @@ getPassword(char *fileName)
 int
 main(int argc, char **argv)
 {
-  char *              progName      = NULL;
-  char *              nickName      = NULL;
-  char *              passwordFile  = NULL;
-  unsigned short      port          = 0;
-  SECStatus           secStatus;
-  PLOptState *        optstate;
-  PLOptStatus         status;
+  const char *    progName      = NULL;
+  const char *    nickName      = NULL;
+  char       *    passwordFile  = NULL;
+  unsigned short  port          = 0;
+  SECStatus       secStatus;
+  PLOptState *    optstate;
+  PLOptStatus     status;
 
   progName = PL_strdup(argv[0]);
 
-  optstate = PL_CreateOptState(argc, argv, "d:p:n:w:");
+  optstate = PL_CreateOptState(argc, argv, "d:p:n:w:s:");
   while ((status = PL_GetNextOpt(optstate)) == PL_OPT_OK)
     {
       switch(optstate->option)
 	{
-	case 'd': dbdir = PL_strdup(optstate->value);           break;
-	case 'n': nickName = PL_strdup(optstate->value);      break;
-	case 'p': port = PORT_Atoi(optstate->value);          break;
-	case 'w': passwordFile = PL_strdup(optstate->value);  break;
+	case 'd': dbdir        = PL_strdup(optstate->value); break;
+	case 'n': nickName     = PL_strdup(optstate->value); break;
+	case 'p': port         = PORT_Atoi(optstate->value); break;
+	case 'w': passwordFile = PL_strdup(optstate->value); break;
+	case 's': stapOptions  = PL_strdup(optstate->value); break;
 	default:
 	case '?': Usage(progName);
 	}
