@@ -1,6 +1,6 @@
-spawn stap args.stp
+spawn stap $srcdir/$subdir/args.stp
 expect {
-    -timeout 240
+    -timeout 60
     "READY" {
 	exec echo 1 > /proc/stap_test_cmd
 	expect {
@@ -53,6 +53,6 @@ expect {
     }
     eof {fail "function arguments: unexpected timeout"}
 }
-send "\003"
-close
+exec kill -INT -[exp_pid]
+catch { close }
 wait
