@@ -9,5 +9,5 @@ lib_func}
 
 # Only run on make installcheck
 if {! [installtest_p]} { untested "lib-$testname"; return }
-if {! [utrace_p]} { untested "lib-$testname"; return }
+if {! [uprobes_p]} { untested "lib-$testname"; return }
 stap_run3 lib-$testname $srcdir/$subdir/lib.stp $testexe $testlib -c $testexe
