@@ -1,5 +1,5 @@
 // -*- C++ -*-
-// Copyright (C) 2005 Red Hat Inc.
+// Copyright (C) 2005, 2009 Red Hat Inc.
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -33,7 +33,7 @@ public:
 
   std::ostream& newline (int indent = 0);
   void indent (int indent = 0);
-  void assert_0_indent () { assert (tablevel == 0); }
+  void assert_0_indent () { o << std::flush; assert (tablevel == 0); }
   std::ostream& line();
 
   std::ostream::pos_type tellp() { return o.tellp(); }
@@ -101,3 +101,5 @@ int translate_pass (systemtap_session& s);
 
 
 #endif // TRANSLATE_H
+
+/* vim: set sw=2 ts=8 cino=>4,n-2,{2,^-2,t0,(0,u0,w1,M1 : */
